/// <reference types="node" />
declare module "IFilter" {
    /**
     * Sync filter information.
     */
    export interface IFilterInfo {
        id: number;
        filter: any;
    }
}
declare module "storage/IStorageProvider" {
    import { IFilterInfo } from "IFilter";
    /**
     * Represents a storage provider for the matrix client
     * @category Storage providers
     */
    export interface IStorageProvider {
        /**
         * Sets the sync token, saving it for later retrieval
         * @param {string} token The token to save
         * @returns {Promise<any>|void} Resolves when complete.
         */
        setSyncToken(token: string | null): Promise<any> | void;
        /**
         * Gets the last saved sync token, or null if none has been persisted.
         * @returns {String|Promise<String>} The last sync token, or null. This can
         * also be a promise for the value.
         */
        getSyncToken(): string | Promise<string | null> | null;
        /**
         * Sets the filter to be used by future clients
         * @param {IFilterInfo} filter The filter to store
         * @returns {Promise<any>|void} Resolves when complete.
         */
        setFilter(filter: IFilterInfo): Promise<any> | void;
        /**
         * Gets the last preferred filter for this client
         * @returns {IFilterInfo|Promise<IFilterInfo>} The last saved filter, or null.
         * This can also be a promise for the filter.
         */
        getFilter(): IFilterInfo | Promise<IFilterInfo>;
        /**
         * Store a simple string value into the provided key.
         * @param {string} key The key to store the value under.
         * @param {string} value The value to store.
         * @returns {Promise<any> | void} Resolves when complete.
         */
        storeValue(key: string, value: string): Promise<any> | void;
        /**
         * Reads a previously stored value under the given key. If the
         * key does not exist, null or undefined is returned.
         * @param {string} key The key to read.
         * @returns {string|Promise<string|null|undefined>|null|undefined} The
         * value, or null/undefined if not found. This may also return a promise
         * of those same values.
         */
        readValue(key: string): string | Promise<string | null | undefined> | null | undefined;
    }
}
declare module "models/events/Event" {
    /**
     * A Matrix event.
     * @category Matrix events
     */
    export class MatrixEvent<T extends (Object | unknown) = unknown> {
        protected event: any;
        constructor(event: any);
        /**
         * The user ID who sent this event.
         */
        get sender(): string;
        /**
         * The type of this event.
         */
        get type(): string;
        /**
         * The content for this event. May have no properties.
         */
        get content(): T;
        /**
         * Gets the raw event that this MatrixEvent is using.
         * Note that there's no guarantees on formats here - it is the exact
         * same input to the constructor.
         */
        get raw(): any;
    }
}
declare module "models/events/RoomEvent" {
    import { MatrixEvent } from "models/events/Event";
    /**
     * The typical unsigned data found on an event.
     * @category Matrix event info
     * @see RoomEvent
     */
    export interface TypicalUnsigned {
        /**
         * The age of this event in seconds.
         */
        age?: number;
        /**
         * Other properties which may be included.
         */
        [prop: string]: any;
    }
    /**
     * Empty room event content.
     * @category Matrix event contents
     */
    export interface RoomEventContent {
    }
    /**
     * A Matrix room event.
     * @category Matrix events
     */
    export class RoomEvent<T extends (Object | unknown) = unknown> extends MatrixEvent<T> {
        protected event: any;
        constructor(event: any);
        /**
         * The event ID of this event.
         */
        get eventId(): string;
        /**
         * The timestamp in milliseconds this event was sent.
         */
        get timestamp(): number;
        /**
         * The unsigned content for this event. May have no properties.
         */
        get unsigned(): TypicalUnsigned;
    }
    /**
     * A room state event.
     * @category Matrix events
     */
    export class StateEvent<T extends (Object | unknown) = unknown> extends RoomEvent<T> {
        constructor(event: any);
        /**
         * The state key for this event. May be an empty string.
         */
        get stateKey(): string;
        /**
         * The previous content for this state event. Will be an empty
         * object if there is no previous content.
         */
        get previousContent(): T;
    }
}
declare module "models/events/EncryptionEvent" {
    import { StateEvent } from "models/events/RoomEvent";
    /**
     * The kinds of room encryption algorithms allowed by the spec.
     * @category Models
     * @see EncryptionEvent
     */
    export enum RoomEncryptionAlgorithm {
        MegolmV1AesSha2 = "m.megolm.v1.aes-sha2"
    }
    /**
     * The content definition for m.room.encryption events
     * @category Matrix event contents
     * @see EncryptionEvent
     */
    export interface EncryptionEventContent {
        /**
         * The encryption algorithm for the room.
         */
        algorithm: string | RoomEncryptionAlgorithm;
        /**
         * How long a session should be used before changing it.
         */
        rotation_period_ms?: number;
        /**
         * How many messages should be sent before changing the session.
         */
        rotation_period_msgs?: number;
    }
    /**
     * Represents an m.room.encryption state event
     * @category Matrix events
     */
    export class EncryptionEvent extends StateEvent<EncryptionEventContent> {
        constructor(event: any);
        /**
         * The encryption algorithm for the room.
         */
        get algorithm(): string | RoomEncryptionAlgorithm;
        /**
         * How long a session should be used before changing it. Defaults to a week.
         */
        get rotationPeriodMs(): number;
        /**
         * How many messages should be sent before a session changes. Defaults to 100.
         */
        get rotationPeriodMessages(): number;
    }
}
declare module "models/Crypto" {
    /**
     * One time key algorithms.
     * @category Models
     */
    export enum OTKAlgorithm {
        Signed = "signed_curve25519",
        Unsigned = "curve25519"
    }
    /**
     * Label for a one time key.
     * @category Models
     */
    export type OTKLabel<Algorithm extends OTKAlgorithm, ID extends string> = `${Algorithm}:${ID}`;
    /**
     * Signatures object.
     * @category Models
     */
    export interface Signatures {
        [entity: string]: {
            [keyLabel: string]: string;
        };
    }
    /**
     * A signed_curve25519 one time key.
     * @category Models
     */
    export interface SignedCurve25519OTK {
        key: string;
        signatures: Signatures;
        fallback?: boolean;
    }
    /**
     * A fallback key.
     * @category Models
     */
    export interface FallbackKey {
        keyId: string;
        key: SignedCurve25519OTK & {
            fallback: true;
        };
    }
    /**
     * One Time Keys structure model.
     * @category Models
     */
    export type OTKs = Record<OTKLabel<OTKAlgorithm.Signed, string>, SignedCurve25519OTK> & Record<OTKLabel<OTKAlgorithm.Unsigned, string>, string>;
    /**
     * The counts of each one time key by algorithm.
     * @category Models
     */
    export type OTKCounts = {
        [alg in OTKAlgorithm]?: number;
    };
    /**
     * The available encryption algorithms.
     * @category Models
     */
    export enum EncryptionAlgorithm {
        OlmV1Curve25519AesSha2 = "m.olm.v1.curve25519-aes-sha2",
        MegolmV1AesSha2 = "m.megolm.v1.aes-sha2"
    }
    /**
     * The key algorithms for device keys.
     * @category Models
     */
    export enum DeviceKeyAlgorithm {
        Ed25519 = "ed25519",
        Curve25519 = "curve25519"
    }
    /**
     * Label for a device key.
     * @category Models
     */
    export type DeviceKeyLabel<Algorithm extends DeviceKeyAlgorithm, ID extends string> = `${Algorithm}:${ID}`;
    /**
     * Represents a user's device.
     * @category Models
     */
    export interface UserDevice {
        user_id: string;
        device_id: string;
        algorithms: (EncryptionAlgorithm | string)[];
        keys: Record<DeviceKeyLabel<DeviceKeyAlgorithm, string>, string>;
        signatures: Signatures;
        unsigned?: {
            [k: string]: any;
            device_display_name?: string;
        };
    }
    /**
     * Represents a user's own device.
     * @category Models
     */
    export interface OwnUserDevice {
        device_id: string;
        display_name?: string;
        last_seen_ip?: string;
        last_seen_ts?: number;
    }
    /**
     * Represents a user's stored device.
     * @category Models
     */
    export interface StoredUserDevice extends UserDevice {
        unsigned: {
            [k: string]: any;
            device_display_name?: string;
            bsdkIsActive: boolean;
        };
    }
    /**
     * Device list response for a multi-user query.
     * @category Models
     */
    export interface MultiUserDeviceListResponse {
        /**
         * Federation failures, keyed by server name. The mapped object should be a standard
         * error object.
         */
        failures: {
            [serverName: string]: any;
        };
        /**
         * A map of user ID to device ID to device.
         */
        device_keys: Record<string, Record<string, UserDevice>>;
    }
    /**
     * One Time Key claim response.
     * @category Models
     */
    export interface OTKClaimResponse {
        /**
         * Federation failures, keyed by server name. The mapped object should be a standard
         * error object.
         */
        failures: {
            [serverName: string]: any;
        };
        /**
         * The claimed One Time Keys, as a map from user ID to device ID to key ID to OTK.
         */
        one_time_keys: Record<string, Record<string, OTKs>>;
    }
    /**
     * An outbound group session.
     * @category Models
     */
    export interface IOutboundGroupSession {
        sessionId: string;
        roomId: string;
        pickled: string;
        isCurrent: boolean;
        usesLeft: number;
        expiresTs: number;
    }
    /**
     * An inbound group session.
     * @category Models
     */
    export interface IInboundGroupSession {
        sessionId: string;
        roomId: string;
        senderUserId: string;
        senderDeviceId: string;
        pickled: string;
    }
    /**
     * An Olm session.
     * @category Models
     */
    export interface IOlmSession {
        sessionId: string;
        pickled: string;
        lastDecryptionTs: number;
    }
    /**
     * An Olm payload (plaintext).
     * @category Models
     */
    export interface IOlmPayload {
        type: string;
        content: any;
        sender: string;
        recipient: string;
        recipient_keys: {
            ed25519: string;
        };
        keys: {
            ed25519: string;
        };
    }
    /**
     * An encrypted Olm payload.
     * @category Models
     */
    export interface IOlmEncrypted {
        algorithm: EncryptionAlgorithm.OlmV1Curve25519AesSha2;
        sender_key: string;
        ciphertext: {
            [deviceCurve25519Key: string]: {
                type: number;
                body: string;
            };
        };
    }
    /**
     * A to-device message.
     * @category Models
     */
    export interface IToDeviceMessage<T = any> {
        type: string;
        sender: string;
        content: T;
    }
    /**
     * An m.room_key to-device message's content.
     * @category Models
     */
    export interface IMRoomKey {
        algorithm: EncryptionAlgorithm.MegolmV1AesSha2;
        room_id: string;
        session_id: string;
        session_key: string;
    }
    /**
     * Encrypted event content for a Megolm-encrypted m.room.encrypted event
     * @category Models
     */
    export interface IMegolmEncrypted {
        algorithm: EncryptionAlgorithm.MegolmV1AesSha2;
        sender_key: string;
        ciphertext: string;
        session_id: string;
        device_id: string;
    }
}
declare module "storage/ICryptoSecureStorageProvider" {
    /**
     * A storage provider for secure information relating to crypto.
     * @category Storage providers
     */
    export interface ICryptoSecureStorageProvider {
        /**
         * Sets the pickle key for the client.
         * @param {string} pickleKey The pickle key to store.
         * @returns {Promise<void>} Resolves when complete.
         */
        setPickleKey(pickleKey: string): Promise<void>;
        /**
         * Gets the pickle key for the client. If no pickle key is set, this resolves
         * to falsy.
         * @returns {Promise<string>} Resolves to the pickle key, or falsy if not set.
         */
        getPickleKey(): Promise<string>;
    }
}
declare module "storage/ICryptoStorageProvider" {
    import { EncryptionEventContent } from "models/events/EncryptionEvent";
    import { IInboundGroupSession, IOlmSession, IOutboundGroupSession, StoredUserDevice, UserDevice } from "models/Crypto";
    import { ICryptoSecureStorageProvider } from "storage/ICryptoSecureStorageProvider";
    /**
     * A storage provider capable of only providing crypto-related storage.
     * @category Storage providers
     */
    export interface ICryptoStorageProvider extends ICryptoSecureStorageProvider {
        /**
         * Sets the client's device ID.
         * @param {string} deviceId The device ID.
         * @returns {Promise<void>} Resolves when complete.
         */
        setDeviceId(deviceId: string): Promise<void>;
        /**
         * Gets the client's device ID, if known.
         * @returns {Promise<string>} Resolves to the device ID, or falsy if not known.
         */
        getDeviceId(): Promise<string>;
        /**
         * Sets the pickled copy of the Olm account. This should be stored securely
         * if possible.
         * @param {string} pickled Encoded, pickled, copy of the Olm account.
         * @returns {Promise<void>} Resolves when complete.
         */
        setPickledAccount(pickled: string): Promise<void>;
        /**
         * Gets the pickled copy of the Olm account, or falsy if not set.
         * @returns {Promise<string>} Resolves to the pickled account, or falsy if not set.
         */
        getPickledAccount(): Promise<string>;
        /**
         * Stores a room's configuration.
         * @param {string} roomId The room ID to store the configuration for.
         * @param {Partial<EncryptionEventContent>} config The room's encryption config. May be empty.
         * @returns {Promise<void>} Resolves when complete.
         */
        storeRoom(roomId: string, config: Partial<EncryptionEventContent>): Promise<void>;
        /**
         * Gets a room's configuration. If the room is unknown, a falsy value is returned.
         * @param {string} roomId The room ID to get the configuration for.
         * @returns {Promise<Partial<EncryptionEventContent>>} Resolves to the room's configuration, or
         * to falsy if the room is unknown.
         */
        getRoom(roomId: string): Promise<Partial<EncryptionEventContent>>;
        /**
         * Sets the user's stored devices to the given array. All devices not in this set will be deleted.
         * This will clear the user's outdated flag, if set.
         * @param {string} userId The user ID to set the devices for.
         * @param {UserDevice[]} devices The devices to set for the user.
         * @returns {Promise<void>} Resolves when complete.
         */
        setActiveUserDevices(userId: string, devices: UserDevice[]): Promise<void>;
        /**
         * Gets the user's active stored devices. If no devices are stored, an empty array is returned.
         * @param {string} userId The user ID to get devices for.
         * @returns {Promise<UserDevice[]>} Resolves to the array of devices for the user. If no
         * devices are known, the array will be empty.
         */
        getActiveUserDevices(userId: string): Promise<UserDevice[]>;
        /**
         * Gets a user's active stored device. If the device is not known or active, falsy is returned.
         * @param {string} userId The user ID.
         * @param {string} deviceId The device ID.
         * @returns {Promise<UserDevice>} Resolves to the user's device, or falsy if not known.
         */
        getActiveUserDevice(userId: string, deviceId: string): Promise<UserDevice>;
        /**
         * Gets all of the user's devices, regardless of whether or not they are active. The active flag
         * will be stored in the unsigned portion of the returned device.
         * @param {string} userId The user ID to get devices for.
         * @returns {Promise<StoredUserDevice[]>} Resolves to the array of devices for the user, or empty
         * if no devices are known.
         */
        getAllUserDevices(userId: string): Promise<StoredUserDevice[]>;
        /**
         * Flags multiple user's device lists as outdated.
         * @param {string} userIds The user IDs to flag.
         * @returns {Promise<void>} Resolves when complete.
         */
        flagUsersOutdated(userIds: string[]): Promise<void>;
        /**
         * Checks to see if a user's device list is flagged as outdated. If the user is not known
         * then they will be considered outdated.
         * @param {string} userId The user ID to check.
         * @returns {Promise<boolean>} Resolves to true if outdated, false otherwise.
         */
        isUserOutdated(userId: string): Promise<boolean>;
        /**
         * Stores a pickled outbound group session. If the session is flagged as current, all other sessions
         * for the room ID will be flagged as not-current.
         * @param {IOutboundGroupSession} session The session to store.
         * @returns {Promise<void>} Resolves when complete.
         */
        storeOutboundGroupSession(session: IOutboundGroupSession): Promise<void>;
        /**
         * Gets a previously stored outbound group session. If the session ID is not known, a falsy value
         * will be returned.
         * @param {string} sessionId The session ID.
         * @param {string} roomId The room ID where the session is stored.
         * @returns {Promise<IOutboundGroupSession>} Resolves to the session, or falsy if not known.
         */
        getOutboundGroupSession(sessionId: string, roomId: string): Promise<IOutboundGroupSession>;
        /**
         * Gets the current outbound group session for a room. If the room does not have a current session,
         * a falsy value will be returned.
         * @param {string} roomId The room ID.
         * @returns {Promise<IOutboundGroupSession>} Resolves to the current session, or falsy if not known.
         */
        getCurrentOutboundGroupSession(roomId: string): Promise<IOutboundGroupSession>;
        /**
         * Stores a session as sent to a user's device.
         * @param {IOutboundGroupSession} session The session that was sent.
         * @param {number} index The session index.
         * @param {UserDevice} device The device the session was sent to.
         * @returns {Promise<void>} Resolves when complete.
         */
        storeSentOutboundGroupSession(session: IOutboundGroupSession, index: number, device: UserDevice): Promise<void>;
        /**
         * Gets the last sent session that was sent to a user's device. If none is recorded,
         * a falsy value is returned.
         * @param {string} userId The user ID to look for.
         * @param {string} deviceId The device ID to look for.
         * @param {string} roomId The room ID to look in.
         * @returns {Promise<{sessionId: string, index: number}>} Resolves to the last session
         * sent, or falsy if not known.
         */
        getLastSentOutboundGroupSession(userId: string, deviceId: string, roomId: string): Promise<{
            sessionId: string;
            index: number;
        }>;
        /**
         * Stores/updates an Olm session for a user's device.
         * @param {string} userId The user ID.
         * @param {string} deviceId The device ID.
         * @param {IOlmSession} session The session.
         * @returns {Promise<void>} Resolves when complete.
         */
        storeOlmSession(userId: string, deviceId: string, session: IOlmSession): Promise<void>;
        /**
         * Gets the most current Olm session for the user's device. If none is present, a falsy value is returned.
         * @param {string} userId The user ID.
         * @param {string} deviceId The device ID.
         * @returns {Promise<IOlmSession>} Resolves to the Olm session, or falsy if none found.
         */
        getCurrentOlmSession(userId: string, deviceId: string): Promise<IOlmSession>;
        /**
         * Gets all the Olm sessions known for a given user's device. Note that this may not return in order,
         * so callers needing to know the "current" Olm session should use the appropriate function.
         * @param {string} userId The user ID.
         * @param {string} deviceId The device ID.
         * @returns {Promise<IOlmSession[]>} Resolves to the known Olm sessions, or an empty array if none are known.
         */
        getOlmSessions(userId: string, deviceId: string): Promise<IOlmSession[]>;
        /**
         * Stores an inbound group session.
         * @param {IInboundGroupSession} session The session to store.
         * @returns {Promise<void>} Resolves when complete.
         */
        storeInboundGroupSession(session: IInboundGroupSession): Promise<void>;
        /**
         * Gets a previously stored inbound group session. If the session is not known, a falsy value is returned.
         * @param {string} senderUserId The user ID who sent the session in the first place.
         * @param {string} senderDeviceId The device ID of the sender.
         * @param {string} roomId The room ID where the session should belong.
         * @param {string} sessionId The session ID itself.
         * @returns {Promise<IInboundGroupSession>} Resolves to the session, or falsy if not known.
         */
        getInboundGroupSession(senderUserId: string, senderDeviceId: string, roomId: string, sessionId: string): Promise<IInboundGroupSession>;
        /**
         * Sets the successfully decrypted message index for an event. Useful for tracking replay attacks.
         * @param {string} roomId The room ID where the event was sent.
         * @param {string} eventId The event ID.
         * @param {string} sessionId The inbound group session ID for the event.
         * @param {number} messageIndex The message index, as reported after decryption.
         * @returns {Promise<void>} Resolves when complete.
         */
        setMessageIndexForEvent(roomId: string, eventId: string, sessionId: string, messageIndex: number): Promise<void>;
        /**
         * Gets the event ID for a previously successful decryption from a session and message index. If
         * no event ID is known, this will return falsy. The caller can use this function to determine if
         * a replay attack is being performed by checking the returned event ID, if present, against the
         * event ID of the event it is decrypting. If the event IDs do not match but are truthy then the
         * session may have been inappropriately re-used.
         * @param {string} roomId The room ID.
         * @param {string} sessionId The inbound group session ID.
         * @param {number} messageIndex The message index.
         * @returns {Promise<string>} Resolves to the event ID of the matching event, or falsy if not known.
         */
        getEventForMessageIndex(roomId: string, sessionId: string, messageIndex: number): Promise<string>;
    }
}
declare module "storage/IAppserviceStorageProvider" {
    import { ICryptoStorageProvider } from "storage/ICryptoStorageProvider";
    import { IStorageProvider } from "storage/IStorageProvider";
    /**
     * A storage provider definition for appservices to use.
     * @category Storage providers
     */
    export interface IAppserviceStorageProvider {
        /**
         * Tracks a user ID as "registered".
         * @returns {Promise<any>|void} Resolves when complete.
         */
        addRegisteredUser(userId: string): Promise<any> | void;
        /**
         * Determines if a user ID is registered or not.
         * @returns {boolean|Promise<boolean>} True if registered. This may be a promise.
         */
        isUserRegistered(userId: string): boolean | Promise<boolean>;
        /**
         * Flags a transaction as completed.
         * @param {string} transactionId The transaction ID.
         * @returns {Promise<any>|void} Resolves when complete.
         */
        setTransactionCompleted(transactionId: string): Promise<any> | void;
        /**
         * Determines if a transaction has been flagged as completed.
         * @param {string} transactionId The transaction ID to check.
         * @returns {boolean} True if the transaction has been completed. This may be a promise.
         */
        isTransactionCompleted(transactionId: string): boolean | Promise<boolean>;
        /**
         * Gets a storage provider to use for the given user ID.
         * @param {string} userId The user ID.
         * @returns {ICryptoStorageProvider} The storage provider.
         */
        storageForUser?(userId: string): IStorageProvider;
    }
    /**
     * A storage provider capable of only providing crypto-related storage to appservices.
     * @category Storage providers
     */
    export interface IAppserviceCryptoStorageProvider {
        /**
         * Gets a storage provider to use for the given user ID.
         * @param {string} userId The user ID.
         * @returns {ICryptoStorageProvider} The storage provider.
         */
        storageForUser(userId: string): ICryptoStorageProvider;
    }
}
declare module "storage/MemoryStorageProvider" {
    import { IStorageProvider } from "storage/IStorageProvider";
    import { IFilterInfo } from "IFilter";
    import { IAppserviceStorageProvider } from "storage/IAppserviceStorageProvider";
    /**
     * A storage provider that persists no information by keeping it all in memory.
     * @category Storage providers
     */
    export class MemoryStorageProvider implements IStorageProvider, IAppserviceStorageProvider {
        private namespaced;
        private syncToken;
        private filter;
        private appserviceUsers;
        private appserviceTransactions;
        private kvStore;
        setSyncToken(token: string | null): void;
        getSyncToken(): string | null;
        setFilter(filter: IFilterInfo): void;
        getFilter(): IFilterInfo;
        addRegisteredUser(userId: string): void;
        isUserRegistered(userId: string): boolean;
        isTransactionCompleted(transactionId: string): boolean;
        setTransactionCompleted(transactionId: string): void;
        readValue(key: string): string | null | undefined;
        storeValue(key: string, value: string): void;
        storageForUser(userId: string): IStorageProvider;
    }
}
declare module "metrics/names" {
    /**
     * Time-series metric for how long a function call takes on MatrixClient. Uses a MatrixClientCallContext.
     * @category Metrics
     */
    export const METRIC_MATRIX_CLIENT_FUNCTION_CALL = "matrix_client_function_call";
    /**
     * Counter metric for failed function calls on a MatrixClient. Uses a MatrixClientCallContext.
     * @category Metrics
     */
    export const METRIC_MATRIX_CLIENT_FAILED_FUNCTION_CALL = "matrix_client_failed_function_call";
    /**
     * Counter metric for successful function calls on a MatrixClient. Uses a MatrixClientCallContext.
     * @category Metrics
     */
    export const METRIC_MATRIX_CLIENT_SUCCESSFUL_FUNCTION_CALL = "matrix_client_successful_function_call";
    /**
     * Time-series metric for how long a function call takes on an IdentityClient. Uses an IdentityClientCallContext.
     * @category Metrics
     */
    export const METRIC_IDENTITY_CLIENT_FUNCTION_CALL = "identity_client_function_call";
    /**
     * Counter metric for failed function calls on an IdentityClient. Uses an IdentityClientCallContext.
     * @category Metrics
     */
    export const METRIC_IDENTITY_CLIENT_FAILED_FUNCTION_CALL = "identity_client_failed_function_call";
    /**
     * Counter metric for successful function calls on an IdentityClient. Uses an IdentityClientCallContext.
     * @category Metrics
     */
    export const METRIC_IDENTITY_CLIENT_SUCCESSFUL_FUNCTION_CALL = "identity_client_successful_function_call";
    /**
     * Time-series metric for how long a function call takes on an Intent. Uses a IntentCallContext.
     * @category Metrics
     */
    export const METRIC_INTENT_FUNCTION_CALL = "intent_function_call";
    /**
     * Counter metric for failed function calls on an Intent. Uses a IntentCallContext.
     * @category Metrics
     */
    export const METRIC_INTENT_FAILED_FUNCTION_CALL = "intent_failed_function_call";
    /**
     * Counter metric for successful function calls on an Intent. Uses a IntentCallContext.
     * @category Metrics
     */
    export const METRIC_INTENT_SUCCESSFUL_FUNCTION_CALL = "intent_successful_function_call";
}
declare module "models/OpenIDConnect" {
    /**
     * An OpenID Connect token from the homeserver.
     * @category Models
     */
    export interface OpenIDConnectToken {
        access_token: string;
        expires_in: number;
        matrix_server_name: string;
        token_type: 'Bearer';
    }
}
declare module "logging/ILogger" {
    /**
     * Represents a logger
     * @category Logging
     */
    export interface ILogger {
        /**
         * Logs to the INFO channel
         * @param {string} module The module being logged
         * @param {any[]} messageOrObject The data to log
         */
        info(module: string, ...messageOrObject: any[]): any;
        /**
         * Logs to the WARN channel
         * @param {string} module The module being logged
         * @param {any[]} messageOrObject The data to log
         */
        warn(module: string, ...messageOrObject: any[]): any;
        /**
         * Logs to the ERROR channel
         * @param {string} module The module being logged
         * @param {any[]} messageOrObject The data to log
         */
        error(module: string, ...messageOrObject: any[]): any;
        /**
         * Logs to the DEBUG channel
         * @param {string} module The module being logged
         * @param {any[]} messageOrObject The data to log
         */
        debug(module: string, ...messageOrObject: any[]): any;
        /**
         * Logs to the TRACE channel
         * @param {string} module The module being logged
         * @param {any[]} messageOrObject The data to log
         */
        trace(module: string, ...messageOrObject: any[]): any;
    }
}
declare module "logging/ConsoleLogger" {
    import { ILogger } from "logging/ILogger";
    /**
     * Logs to the console in a plain format. This is the default logger.
     * @category Logging
     */
    export class ConsoleLogger implements ILogger {
        trace(module: string, ...messageOrObject: any[]): void;
        debug(module: string, ...messageOrObject: any[]): void;
        error(module: string, ...messageOrObject: any[]): void;
        info(module: string, ...messageOrObject: any[]): void;
        warn(module: string, ...messageOrObject: any[]): void;
    }
}
declare module "logging/LogService" {
    import { ILogger } from "logging/ILogger";
    /**
     * The log levels to log at.
     * @category Logging
     */
    export class LogLevel {
        private level;
        private sequence;
        /**
         * The TRACE channel
         */
        static readonly TRACE: LogLevel;
        /**
         * The DEBUG channel
         */
        static readonly DEBUG: LogLevel;
        /**
         * The INFO channel
         */
        static readonly INFO: LogLevel;
        /**
         * The WARN channel
         */
        static readonly WARN: LogLevel;
        /**
         * The ERROR channel
         */
        static readonly ERROR: LogLevel;
        private constructor();
        includes(level: LogLevel): boolean;
        toString(): string;
        static fromString(level: string, defaultLevel?: LogLevel): LogLevel;
    }
    /**
     * Service class for logging in the bot-sdk
     * @category Logging
     */
    export class LogService {
        private static logger;
        private static logLevel;
        private static mutedModules;
        private constructor();
        /**
         * The level at which the LogService is running.
         */
        static get level(): LogLevel;
        /**
         * Sets the log level for this logger. Defaults to DEBUG.
         * @param {LogLevel} level the new log level
         */
        static setLevel(level: LogLevel): void;
        /**
         * Sets a new logger for the Log Service
         * @param {ILogger} logger the new logger
         */
        static setLogger(logger: ILogger): void;
        /**
         * Mutes a module from the logger.
         * @param {string} name The module name to mute.
         */
        static muteModule(name: string): void;
        /**
         * Logs to the TRACE channel
         * @param {string} module The module being logged
         * @param {any[]} messageOrObject The data to log
         */
        static trace(module: string, ...messageOrObject: any[]): void;
        /**
         * Logs to the DEBUG channel
         * @param {string} module The module being logged
         * @param {any[]} messageOrObject The data to log
         */
        static debug(module: string, ...messageOrObject: any[]): void;
        /**
         * Logs to the ERROR channel
         * @param {string} module The module being logged
         * @param {any[]} messageOrObject The data to log
         */
        static error(module: string, ...messageOrObject: any[]): void;
        /**
         * Logs to the INFO channel
         * @param {string} module The module being logged
         * @param {any[]} messageOrObject The data to log
         */
        static info(module: string, ...messageOrObject: any[]): void;
        /**
         * Logs to the WARN channel
         * @param {string} module The module being logged
         * @param {any[]} messageOrObject The data to log
         */
        static warn(module: string, ...messageOrObject: any[]): void;
    }
    /**
     * Extracts the useful part of a request's error into something loggable.
     * @param {Error} err The error to parse.
     * @returns {*} The extracted error, or the given error if unaltered.
     * @category Logging
     */
    export function extractRequestError(err: Error): any;
}
declare module "request" {
    /**
     * Sets the function to use for performing HTTP requests. Must be compatible with `request`.
     * @param fn The new request function.
     * @category Unit testing
     */
    export function setRequestFn(fn: any): void;
    /**
     * Gets the `request`-compatible function for performing HTTP requests.
     * @returns The request function.
     * @category Unit testing
     */
    export function getRequestFn(): any;
}
declare module "http" {
    /**
     * Performs a web request to a server.
     * @category Unit testing
     * @param {string} baseUrl The base URL to apply to the call.
     * @param {"GET"|"POST"|"PUT"|"DELETE"} method The HTTP method to use in the request
     * @param {string} endpoint The endpoint to call. For example: "/_matrix/client/r0/account/whoami"
     * @param {any} qs The query string to send. Optional.
     * @param {any} body The request body to send. Optional. Will be converted to JSON unless the type is a Buffer.
     * @param {any} headers Additional headers to send in the request.
     * @param {number} timeout The number of milliseconds to wait before timing out.
     * @param {boolean} raw If true, the raw response will be returned instead of the response body.
     * @param {string} contentType The content type to send. Only used if the `body` is a Buffer.
     * @param {string} noEncoding Set to true to disable encoding, and return a Buffer. Defaults to false
     * @returns {Promise<any>} Resolves to the response (body), rejected if a non-2xx status code was returned.
     */
    export function doHttpRequest(baseUrl: string, method: "GET" | "POST" | "PUT" | "DELETE", endpoint: string, qs?: any, body?: any, headers?: {}, timeout?: number, raw?: boolean, contentType?: string, noEncoding?: boolean): Promise<any>;
    export function redactObjectForLogging(input: any): any;
}
declare module "models/Policies" {
    /**
     * Information about the policies (terms of service) a server may have.
     * @category Models
     */
    export interface Policies {
        policies: {
            [id: string]: Policy;
        };
    }
    /**
     * Information about a policy (terms of service) a server may have.
     * @category Models
     */
    export interface Policy {
        version: string;
        [language: string]: TranslatedPolicy | string;
    }
    /**
     * Information about a (translated) policy (terms of service).
     * @category Models
     */
    export interface TranslatedPolicy {
        name: string;
        url: string;
    }
}
declare module "metrics/IMetricListener" {
    import { IMetricContext } from "metrics/contexts";
    /**
     * A simple interface for listening for Metric updates. Should be plugged into
     * something like Prometheus for recording.
     *
     * Metric names are defined in metric_names.ts - see documentation on the name
     * for what the context object contains. All metrics have a context object,
     * with applicable interface. See the IMetricContext interface for more
     * information.
     * @category Metrics
     */
    export interface IMetricListener {
        /**
         * Called when the given metric should start being tracked. Will be
         * paired with a matching onEndMetric() call.
         * @param {string} metricName The metric being called.
         * @param {IMetricContext} context Context for the metric. Never null.
         */
        onStartMetric(metricName: string, context: IMetricContext): void;
        /**
         * Called when the given metric should stop being tracked. Will have
         * started with a matching onStartMetric() call.
         * @param {string} metricName The metric being called.
         * @param {any} context Context for the metric. Never null.
         * @param {number} timeMs The measured time in milliseconds between
         * the start and end.
         */
        onEndMetric(metricName: string, context: IMetricContext, timeMs: number): void;
        /**
         * Called when a linear metric (increasing/decreasing number) should
         * be incremented.
         * @param {string} metricName The metric being called.
         * @param {IMetricContext} context Context for the metric. Never null.
         * @param {number} amount The amount to add. Never negative or zero.
         */
        onIncrement(metricName: string, context: IMetricContext, amount: number): any;
        /**
         * Called when a linear metric (increasing/decreasing number) should
         * be decremented.
         * @param {string} metricName The metric being called.
         * @param {IMetricContext} context Context for the metric. Never null.
         * @param {number} amount The amount to subtract. Never negative or zero.
         */
        onDecrement(metricName: string, context: IMetricContext, amount: number): any;
        /**
         * Called when a linear metric (increasing/decreasing number) should
         * be reset to zero.
         * @param {string} metricName The metric being called.
         * @param {IMetricContext} context Context for the metric. Never null.
         */
        onReset(metricName: string, context: IMetricContext): any;
    }
}
declare module "metrics/Metrics" {
    import { IMetricListener } from "metrics/IMetricListener";
    import { IMetricContext } from "metrics/contexts";
    /**
     * Tracks metrics.
     * @category Metrics
     */
    export class Metrics {
        private listeners;
        private requestStartTimes;
        private uid;
        /**
         * Creates a new Metrics handler with optional parent handler. When
         * a parent handler is defined, metrics will be automatically published
         * upwards to the parent.
         * @param {Metrics} parent Optional parent for upstream metrics.
         */
        constructor(parent?: Metrics);
        /**
         * Registers a metric listener.
         * @param {IMetricListener} listener The listener.
         */
        registerListener(listener: IMetricListener): void;
        /**
         * De-registers a metric listener.
         * @param {IMetricListener} listener The listener.
         */
        unregisterListener(listener: IMetricListener): void;
        /**
         * Starts a timer on a metric.
         * @param {string} metricName The metric name.
         * @param {IMetricContext} context The metric context. Expected to have a unique ID.
         */
        start(metricName: string, context: IMetricContext): void;
        /**
         * Ends a timer on a metric.
         * @param {string} metricName The metric name.
         * @param {IMetricContext} context The metric context. Expected to have a unique ID.
         */
        end(metricName: string, context: IMetricContext): void;
        /**
         * Increments a metric.
         * @param {string} metricName The metric name.
         * @param {IMetricContext} context The metric context. Expected to have a unique ID.
         * @param {number} amount The amount.
         */
        increment(metricName: string, context: IMetricContext, amount: number): void;
        /**
         * Decrements a metric.
         * @param {string} metricName The metric name.
         * @param {IMetricContext} context The metric context. Expected to have a unique ID.
         * @param {number} amount The amount.
         */
        decrement(metricName: string, context: IMetricContext, amount: number): void;
        /**
         * Resets a metric.
         * @param {string} metricName The metric name.
         * @param {IMetricContext} context The metric context. Expected to have a unique ID.
         */
        reset(metricName: string, context: IMetricContext): void;
        /**
         * Assigns a unique ID to the context object, returning it back.
         * @param {IMetricContext} context The context to modify.
         * @returns {IMetricContext} The provided context.
         */
        assignUniqueContextId(context: IMetricContext): IMetricContext;
    }
}
declare module "models/Threepid" {
    /**
     * A Third Party Identifier (3PID or threepid)
     * @category Models
     */
    export interface Threepid {
        kind: "email" | "msisdn" | string;
        address: string;
    }
}
declare module "helpers/UnpaddedBase64" {
    /**
     * Unpadded Base64 utilities for Matrix.
     * @category Utilities
     */
    export class UnpaddedBase64 {
        private constructor();
        /**
         * Encodes a buffer to Unpadded Base64
         * @param {Buffer} buf The buffer to encode.
         * @returns {string} The Unpadded Base64 string
         */
        static encodeBuffer(buf: Buffer): string;
        /**
         * Encodes a string to Unpadded Base64
         * @param {string} str The string to encode.
         * @returns {string} The Unpadded Base64 string
         */
        static encodeString(str: string): string;
        /**
         * Encodes a buffer to Unpadded Base64 (URL Safe Edition)
         * @param {Buffer} buf The buffer to encode.
         * @returns {string} The Unpadded Base64 string
         */
        static encodeBufferUrlSafe(buf: Buffer): string;
        /**
         * Encodes a string to Unpadded Base64 (URL Safe Edition)
         * @param {string} str The string to encode.
         * @returns {string} The Unpadded Base64 string
         */
        static encodeStringUrlSafe(str: string): string;
    }
}
declare module "models/MatrixProfile" {
    import { MentionPill } from "index";
    /**
     * Profile information commonly associated with Matrix profiles
     * @category Models
     */
    export interface MatrixProfileInfo {
        /**
         * The display name of the user, if any.
         */
        displayname?: string;
        /**
         * A URL to the user's avatar, if any.
         */
        avatar_url?: string;
    }
    /**
     * Represents a user's profile, possibly in a room.
     * @category Models
     */
    export class MatrixProfile {
        private userId;
        private profile;
        /**
         * Creates a new profile representation for a user.
         * @param {string} userId The user ID the profile is for.
         * @param {MatrixProfile} profile The profile itself.
         */
        constructor(userId: string, profile: MatrixProfileInfo);
        /**
         * The display name for the user. This will always return a value, though it
         * may be based upon their user ID if no explicit display name is set.
         */
        get displayName(): string;
        /**
         * The avatar URL for the user. If the user does not have an avatar, this will
         * be null.
         */
        get avatarUrl(): string;
        /**
         * A mention pill for this user.
         */
        get mention(): MentionPill;
    }
}
declare module "models/IdentityServerModels" {
    /**
     * Information about a user on an identity server.
     * @category Models
     */
    export interface IdentityServerAccount {
        user_id: string;
    }
    /**
     * A stored invite on an identity server.
     * @category Models
     */
    export interface IdentityServerInvite {
        display_name: string;
        public_keys: {
            public_key: string;
            key_validity_url: string;
        }[];
        public_key: string;
        token: string;
    }
}
declare module "identity/IdentityClient" {
    import { OpenIDConnectToken } from "models/OpenIDConnect";
    import { Policies } from "models/Policies";
    import { Metrics } from "metrics/Metrics";
    import { Threepid } from "models/Threepid";
    import { MatrixClient } from "MatrixClient";
    import { IdentityServerAccount, IdentityServerInvite } from "models/IdentityServerModels";
    /**
     * A way to access an Identity Server using the Identity Service API from a MatrixClient.
     * @category Identity Servers
     */
    export class IdentityClient {
        readonly accessToken: string;
        readonly serverUrl: string;
        readonly matrixClient: MatrixClient;
        /**
         * The metrics instance for this client. Note that metrics for the underlying MatrixClient will
         * not be available here.
         */
        readonly metrics: Metrics;
        /**
         * If truthy, this is a string that will be supplied as `?brand=$brand` where endpoints can
         * result in communications to a user.
         */
        brand: string;
        private constructor();
        /**
         * Gets account information for the logged in user.
         * @returns {Promise<IdentityServerAccount>} Resolves to the account information
         */
        getAccount(): Promise<IdentityServerAccount>;
        /**
         * Gets the terms of service for which the identity server has.
         * @returns {Promise<Policies>} Resolves to the policies of the server.
         */
        getTermsOfService(): Promise<Policies>;
        /**
         * Accepts a given set of URLs from Policy objects returned by the server. This implies acceptance of
         * the terms. Note that this will not update the user's account data to consider these terms accepted
         * in the future - that is an exercise left to the caller.
         * @param {string[]} termsUrls The URLs to count as accepted.
         * @returns {Promise<void>} Resolves when complete.
         */
        acceptTerms(termsUrls: string[]): Promise<void>;
        /**
         * Accepts all the terms of service offered by the identity server. Note that this is only meant to be
         * used by automated bots where terms acceptance is implicit - the terms of service need to be presented
         * to the user in most cases.
         * @returns {Promise<void>} Resolves when complete.
         */
        acceptAllTerms(): Promise<void>;
        /**
         * Looks up a series of third party identifiers (email addresses or phone numbers) to see if they have
         * associated mappings. The returned array will be ordered the same as the input, and have falsey values
         * in place of any failed/missing lookups (eg: no mapping).
         * @param {Threepid[]} identifiers The identifiers to look up.
         * @param {boolean} allowPlaintext If true, the function will accept the server's offer to use plaintext
         * lookups when no other methods are available. The function will always prefer hashed methods.
         * @returns {Promise<string[]>} Resolves to the user IDs (or falsey values) in the same order as the input.
         */
        lookup(identifiers: Threepid[], allowPlaintext?: boolean): Promise<string[]>;
        /**
         * Creates a third party email invite. This will store the invite in the identity server, but
         * not publish the invite to the room - the caller is expected to handle the remaining part. Note
         * that this function is not required to be called when using the Client-Server API for inviting
         * third party addresses to a room. This will make several calls into the room state to populate
         * the invite details, therefore the inviter (the client backing this identity client) must be
         * present in the room.
         * @param {string} emailAddress The email address to invite.
         * @param {string} roomId The room ID to invite to.
         * @returns {Promise<IdentityServerInvite>} Resolves to the identity server's stored invite.
         */
        makeEmailInvite(emailAddress: string, roomId: string): Promise<IdentityServerInvite>;
        /**
         * Performs a web request to the server, applying appropriate authorization headers for
         * this client.
         * @param {"GET"|"POST"|"PUT"|"DELETE"} method The HTTP method to use in the request
         * @param {string} endpoint The endpoint to call. For example: "/_matrix/identity/v2/account"
         * @param {any} qs The query string to send. Optional.
         * @param {any} body The request body to send. Optional. Will be converted to JSON unless the type is a Buffer.
         * @param {number} timeout The number of milliseconds to wait before timing out.
         * @param {boolean} raw If true, the raw response will be returned instead of the response body.
         * @param {string} contentType The content type to send. Only used if the `body` is a Buffer.
         * @param {string} noEncoding Set to true to disable encoding, and return a Buffer. Defaults to false
         * @returns {Promise<any>} Resolves to the response (body), rejected if a non-2xx status code was returned.
         */
        doRequest(method: any, endpoint: any, qs?: any, body?: any, timeout?: number, raw?: boolean, contentType?: string, noEncoding?: boolean): Promise<any>;
        /**
         * Gets an instance of an identity client.
         * @param {OpenIDConnectToken} oidc The OpenID Connect token to register to the identity server with.
         * @param {string} serverUrl The full URL where the identity server can be reached at.
         */
        static acquire(oidc: OpenIDConnectToken, serverUrl: string, mxClient: MatrixClient): Promise<IdentityClient>;
    }
}
declare module "metrics/contexts" {
    /**
     * Default context for all metrics.
     * @category Metrics
     */
    import { MatrixClient } from "MatrixClient";
    import { Intent } from "appservice/Intent";
    import { IdentityClient } from "identity/IdentityClient";
    export interface IMetricContext {
        /**
         * Unique identifier for the context object. Used to differentiate
         * contexts over a start/end event.
         */
        uniqueId: string;
    }
    /**
     * Metric context for function call metrics.
     * @category Metrics
     */
    export interface FunctionCallContext extends IMetricContext {
        /**
         * The function name being called
         */
        functionName: string;
    }
    /**
     * Metric context for metrics from a MatrixClient
     * @category Metrics
     */
    export interface MatrixClientCallContext extends FunctionCallContext {
        /**
         * The client that raised the metric.
         */
        client: MatrixClient;
    }
    /**
     * Metric context for metrics from an IdentityClient
     * @category Metrics
     */
    export interface IdentityClientCallContext extends FunctionCallContext {
        client: IdentityClient;
    }
    /**
     * Metric context for metrics from an Intent
     * @category Metrics
     */
    export interface IntentCallContext extends MatrixClientCallContext {
        /**
         * The intent that is raising the metric.
         */
        intent: Intent;
    }
}
declare module "metrics/decorators" {
    /**
     * Times a MatrixClient function call for metrics.
     * @category Metrics
     */
    export function timedMatrixClientFunctionCall(): (target: any, propertyKey: string, descriptor: PropertyDescriptor) => void;
    /**
     * Times an IdentityClient function call for metrics.
     * @category Metrics
     */
    export function timedIdentityClientFunctionCall(): (target: any, propertyKey: string, descriptor: PropertyDescriptor) => void;
    /**
     * Times an Intent function call for metrics.
     * @category Metrics
     */
    export function timedIntentFunctionCall(): (target: any, propertyKey: string, descriptor: PropertyDescriptor) => void;
}
declare module "models/MSC2176" {
    /**
     * Response object for a batch send operation.
     * @category Models
     */
    export interface MSC2716BatchSendResponse {
        /**
         * List of historical state event IDs that were inserted
         */
        state_events?: string[];
        /**
         * List of historical event IDs that were inserted
         */
        events?: string[];
        /**
         * Chunk ID to be used in the next `sendHistoricalEventBatch` call.
         */
        next_chunk_id: string;
    }
    /**
     * Partial event content for an inserted MSC2716 event.
     * @category Matrix event contents
     */
    export interface MSC2716InsertionEventContent {
        "org.matrix.msc2716.next_chunk_id": string;
        "org.matrix.msc2716.historical": true;
    }
    /**
     * Partial event content for a chunked MSC2716 event.
     * @category Matrix event contents
     */
    export interface MSC2716ChunkEventContent {
        "org.matrix.msc2716.chunk_id": string;
        "org.matrix.msc2716.historical": true;
    }
    /**
     * Partial event content for a marked MSC2716 event.
     * @category Matrix event contents
     */
    export interface MSC2716MarkerEventContent {
        "org.matrix.msc2716.insertion_id": string;
        "org.matrix.msc2716.historical": true;
    }
}
declare module "appservice/UnstableAppserviceApis" {
    import { MatrixClient } from "MatrixClient";
    import { MSC2716BatchSendResponse } from "models/MSC2176";
    /**
     * Unstable APIs that shouldn't be used in most circumstances for appservices.
     * @category Unstable APIs
     */
    export class UnstableAppserviceApis {
        private client;
        private requestId;
        constructor(client: MatrixClient);
        /**
         * Send several historical events into a room.
         * @see https://github.com/matrix-org/matrix-doc/pull/2716
         * @param {string} roomId The roomID to send to.
         * @param {string} prevEventId The event ID where this batch will be inserted
         * @param {string} chunkId The chunk ID returned from a previous call. Set falsy to start at the beginning.
         * @param {any[]} events A set of event contents for events to be inserted into the room.
         * @param {any[]} stateEventsAtStart A set of state events to be inserted into the room. Defaults to empty.
         * @returns A set of eventIds and the next chunk ID
         */
        sendHistoricalEventBatch(roomId: string, prevEventId: string, events: any[], stateEventsAtStart?: any[], chunkId?: string): Promise<MSC2716BatchSendResponse>;
        /**
         * Sends an event to the given room with a given timestamp.
         * @param {string} roomId the room ID to send the event to
         * @param {string} eventType the type of event to send
         * @param {string} content the event body to send
         * @param {number} ts The origin_server_ts of the new event
         * @returns {Promise<string>} resolves to the event ID that represents the event
         */
        sendEventWithTimestamp(roomId: string, eventType: string, content: any, ts: number): Promise<any>;
        /**
         * Sends a state event to the given room with a given timestamp.
         * @param {string} roomId the room ID to send the event to
         * @param {string} type the event type to send
         * @param {string} stateKey the state key to send, should not be null
         * @param {string} content the event body to send
         * @param {number} ts The origin_server_ts of the new event
         * @returns {Promise<string>} resolves to the event ID that represents the message
         */
        sendStateEventWithTimestamp(roomId: string, type: string, stateKey: string, content: any, ts: number): Promise<string>;
    }
}
declare module "appservice/Intent" {
    import { MatrixClient, Metrics } from "index";
    import { Appservice, IAppserviceOptions } from "appservice/Appservice";
    import { UnstableAppserviceApis } from "appservice/UnstableAppserviceApis";
    /**
     * An Intent is an intelligent client that tracks things like the user's membership
     * in rooms to ensure the action being performed is possible. This is very similar
     * to how Intents work in the matrix-js-sdk in that the Intent will ensure that the
     * user is joined to the room before posting a message, for example.
     * @category Application services
     */
    export class Intent {
        private options;
        private impersonateUserId;
        private appservice;
        /**
         * The metrics instance for this intent. Note that this will not raise metrics
         * for the underlying client - those will be available through this instance's
         * parent (the appservice).
         */
        readonly metrics: Metrics;
        private readonly storage;
        private readonly cryptoStorage;
        private client;
        private unstableApisInstance;
        private knownJoinedRooms;
        private cryptoSetupPromise;
        /**
         * Creates a new intent. Intended to be created by application services.
         * @param {IAppserviceOptions} options The options for the application service.
         * @param {string} impersonateUserId The user ID to impersonate.
         * @param {Appservice} appservice The application service itself.
         */
        constructor(options: IAppserviceOptions, impersonateUserId: string, appservice: Appservice);
        private makeClient;
        /**
         * Gets the user ID this intent is for.
         */
        get userId(): string;
        /**
         * Gets the underlying MatrixClient that powers this Intent.
         */
        get underlyingClient(): MatrixClient;
        /**
         * Gets the unstable API access class. This is generally not recommended to be
         * used by appservices.
         * @return {UnstableAppserviceApis} The unstable API access class.
         */
        get unstableApis(): UnstableAppserviceApis;
        /**
         * Sets up crypto on the client if it hasn't already been set up.
         * @returns {Promise<void>} Resolves when complete.
         */
        enableEncryption(): Promise<void>;
        /**
         * Gets the joined rooms for the intent. Note that by working around
         * the intent to join rooms may yield inaccurate results.
         * @returns {Promise<string[]>} Resolves to an array of room IDs where
         * the intent is joined.
         */
        getJoinedRooms(): Promise<string[]>;
        /**
         * Leaves the given room.
         * @param {string} roomId The room ID to leave
         * @returns {Promise<any>} Resolves when the room has been left.
         */
        leaveRoom(roomId: string): Promise<any>;
        /**
         * Joins the given room
         * @param {string} roomIdOrAlias the room ID or alias to join
         * @returns {Promise<string>} resolves to the joined room ID
         */
        joinRoom(roomIdOrAlias: string): Promise<string>;
        /**
         * Sends a text message to a room.
         * @param {string} roomId The room ID to send text to.
         * @param {string} body The message body to send.
         * @param {"m.text" | "m.emote" | "m.notice"} msgtype The message type to send.
         * @returns {Promise<string>} Resolves to the event ID of the sent message.
         */
        sendText(roomId: string, body: string, msgtype?: "m.text" | "m.emote" | "m.notice"): Promise<string>;
        /**
         * Sends an event to a room.
         * @param {string} roomId The room ID to send the event to.
         * @param {any} content The content of the event.
         * @returns {Promise<string>} Resolves to the event ID of the sent event.
         */
        sendEvent(roomId: string, content: any): Promise<string>;
        /**
         * Ensures the user is registered and joined to the given room.
         * @param {string} roomId The room ID to join
         * @returns {Promise<any>} Resolves when complete
         */
        ensureRegisteredAndJoined(roomId: string): Promise<void>;
        /**
         * Ensures the user is joined to the given room
         * @param {string} roomId The room ID to join
         * @returns {Promise<any>} Resolves when complete
         */
        ensureJoined(roomId: string): Promise<string>;
        /**
         * Refreshes which rooms the user is joined to, potentially saving time on
         * calls like ensureJoined()
         * @returns {Promise<string[]>} Resolves to the joined room IDs for the user.
         */
        refreshJoinedRooms(): Promise<string[]>;
        /**
         * Ensures the user is registered
         * @returns {Promise<any>} Resolves when complete
         */
        ensureRegistered(): Promise<any>;
    }
}
declare module "appservice/MatrixBridge" {
    import { Appservice } from "appservice/Appservice";
    import { Intent } from "appservice/Intent";
    export const REMOTE_USER_INFO_ACCOUNT_DATA_EVENT_TYPE = "io.t2bot.sdk.bot.remote_user_info";
    export const REMOTE_ROOM_INFO_ACCOUNT_DATA_EVENT_TYPE = "io.t2bot.sdk.bot.remote_room_info";
    export const REMOTE_USER_MAP_ACCOUNT_DATA_EVENT_TYPE_PREFIX = "io.t2bot.sdk.bot.remote_user_map";
    export const REMOTE_ROOM_MAP_ACCOUNT_DATA_EVENT_TYPE_PREFIX = "io.t2bot.sdk.bot.remote_room_map";
    /**
     * @see MatrixBridge
     * @category Application services
     */
    export interface IRemoteRoomInfo {
        /**
         * A unique identifier for the remote user.
         */
        id: string;
    }
    /**
     * @see MatrixBridge
     * @category Application services
     */
    export interface IRemoteUserInfo {
        /**
         * A unique identifier for the remote room (or room equivalent).
         */
        id: string;
    }
    /**
     * Utility class for common operations performed by bridges (represented
     * as appservices).
     *
     * The storage utilities are not intended for bridges which allow 1:many
     * relationships with the remote network.
     *
     * Bridges are generally expected to create their own classes which extend
     * the IRemoteRoomInfo and IRemoteUserInfo interfaces and serialize to JSON
     * cleanly. The serialized version of these classes is persisted in various
     * account data locations for future lookups.
     * @category Application services
     */
    export class MatrixBridge {
        private appservice;
        constructor(appservice: Appservice);
        /**
         * Gets information about a remote user.
         * @param {Intent} userIntent The Matrix user intent to get information on.
         * @returns {Promise<IRemoteUserInfo>} Resolves to the remote user information.
         */
        getRemoteUserInfo<T extends IRemoteUserInfo>(userIntent: Intent): Promise<T>;
        /**
         * Sets information about a remote user. Calling this function will map the
         * provided remote user ID to the intent's owner.
         * @param {Intent} userIntent The Matrix user intent to store information on.
         * @param {IRemoteUserInfo} remoteInfo The remote user information to store
         * @returns {Promise<any>} Resolves when the information has been updated.
         */
        setRemoteUserInfo<T extends IRemoteUserInfo>(userIntent: Intent, remoteInfo: T): Promise<any>;
        /**
         * Gets information about a remote room.
         * @param {string} matrixRoomId The Matrix room ID to get information on.
         * @returns {Promise<IRemoteRoomInfo>} Resolves to the remote room information.
         */
        getRemoteRoomInfo<T extends IRemoteRoomInfo>(matrixRoomId: string): Promise<T>;
        /**
         * Sets information about a remote room. Calling this function will map the
         * provided remote room ID to the matrix room ID.
         * @param {string} matrixRoomId The Matrix room ID to store information on.
         * @param {IRemoteRoomInfo} remoteInfo The remote room information to store
         * @returns {Promise<any>} Resolves when the information has been updated.
         */
        setRemoteRoomInfo<T extends IRemoteRoomInfo>(matrixRoomId: string, remoteInfo: T): Promise<any>;
        /**
         * Gets the Matrix room ID for the provided remote room ID.
         * @param {string} remoteRoomId The remote room ID to look up.
         * @returns {Promise<string>} Resolves to the Matrix room ID.
         */
        getMatrixRoomIdForRemote(remoteRoomId: string): Promise<string>;
        /**
         * Gets a Matrix user intent for the provided remote user ID.
         * @param {string} remoteUserId The remote user ID to look up.
         * @returns {Promise<Intent>} Resolves to the Matrix user intent.
         */
        getIntentForRemote(remoteUserId: string): Promise<Intent>;
        private updateRemoteUserMapping;
        private updateRemoteRoomMapping;
    }
}
declare module "appservice/http_responses" {
    /**
     * This is the response format documented in
     * https://matrix.org/docs/spec/application_service/r0.1.2#get-matrix-app-v1-thirdparty-protocol-protocol
     * @category Application services
     */
    export interface IApplicationServiceProtocol {
        user_fields: string[];
        location_fields: string[];
        icon: string;
        field_types: {
            [field: string]: IFieldType;
        };
        instances: {
            [name: string]: IProtocolInstance;
        };
    }
    interface IFieldType {
        regexp: string;
        placeholder: string;
    }
    interface IProtocolInstance {
        desc: string;
        icon: string;
        fields: {
            [field: string]: string;
        };
        network_id: string;
    }
}
declare module "appservice/Appservice" {
    import { Intent } from "appservice/Intent";
    import { IAppserviceCryptoStorageProvider, IAppserviceStorageProvider, IJoinRoomStrategy, IPreprocessor, MatrixClient, Metrics } from "index";
    import { EventEmitter } from "events";
    import { MatrixBridge } from "appservice/MatrixBridge";
    /**
     * Represents an application service's registration file. This is expected to be
     * loaded from another source, such as a YAML file.
     * @category Application services
     */
    export interface IAppserviceRegistration {
        /**
         * Optional ID for the appplication service. Used by homeservers to track which application
         * service registers what.
         */
        id?: string;
        /**
         * Optional URL at which the application service can be contacted.
         */
        url?: string;
        /**
         * The token the application service uses to communicate with the homeserver.
         */
        as_token: string;
        /**
         * The token the homeserver uses to communicate with the application service.
         */
        hs_token: string;
        /**
         * The application service's own localpart (eg: "_irc_bot" in the user ID "@_irc_bot:domain.com")
         */
        sender_localpart: string;
        /**
         * The various namespaces the application service can support.
         */
        namespaces: {
            /**
             * The user namespaces the application service is requesting.
             */
            users: {
                /**
                 * Whether or not the application service holds an exclusive lock on the namespace. This
                 * means that no other user on the homeserver may register users that match this namespace.
                 */
                exclusive: boolean;
                /**
                 * The regular expression that the homeserver uses to determine if a user is in this namespace.
                 */
                regex: string;
                /**
                 * An optional group ID to enable flair for users in this namespace.
                 */
                groupId?: string;
            }[];
            /**
             * The room namespaces the application service is requesting. This is not for alises.
             */
            rooms: {
                /**
                 * Whether or not the application service holds an exclusive lock on the namespace.
                 */
                exclusive: boolean;
                /**
                 * The regular expression that the homeserver uses to determine if a user is in this namespace.
                 */
                regex: string;
            }[];
            /**
             * The room alias namespaces the application service is requesting.
             */
            aliases: {
                /**
                 * Whether or not the application service holds an exclusive lock on the namespace. This means
                 * that no other user on the homeserver may register aliases that match this namespace.
                 */
                exclusive: boolean;
                /**
                 * The regular expression that the homeserver uses to determine if an alias is in this namespace.
                 */
                regex: string;
            }[];
        };
        /**
         * The protocols the application service supports. Optional.
         */
        protocols?: string[];
        /**
         * If the application service is rate limited by the homeserver. Optional.
         */
        rate_limited?: boolean;
        /**
         * **Experimental**
         *
         * Should the application service receive ephemeral events from the homeserver. Optional.
         * @see https://github.com/matrix-org/matrix-doc/pull/2409
         */
        "de.sorunome.msc2409.push_ephemeral"?: boolean;
    }
    /**
     * General options for the application service
     * @category Application services
     */
    export interface IAppserviceOptions {
        /**
         * The port to listen for requests from the homeserver on.
         */
        port: number;
        /**
         * The bind address to listen for requests on.
         */
        bindAddress: string;
        /**
         * The name of the homeserver, as presented over federation (eg: "matrix.org")
         */
        homeserverName: string;
        /**
         * The URL to the homeserver's client server API (eg: "https://matrix.org")
         */
        homeserverUrl: string;
        /**
         * The storage provider to use for this application service.
         */
        storage?: IAppserviceStorageProvider;
        /**
         * The storage provider to use for setting up encryption. Encryption will be
         * disabled for all intents and the appservice if not configured.
         */
        cryptoStorage?: IAppserviceCryptoStorageProvider;
        /**
         * The registration for this application service.
         */
        registration: IAppserviceRegistration;
        /**
         * The join strategy to use for all intents, if any.
         */
        joinStrategy?: IJoinRoomStrategy;
        /**
         * Options for how Intents are handled.
         */
        intentOptions?: {
            /**
             * The maximum number of intents to keep cached. Defaults to 10 thousand.
             */
            maxCached?: number;
            /**
             * The maximum age in milliseconds to keep an Intent around for, provided
             * the maximum number of intents has been reached. Defaults to 60 minutes.
             */
            maxAgeMs?: number;
            /**
             * If false (default), crypto will not be automatically set up for all intent
             * instances - it will need to be manually enabled with
             * `await intent.enableEncryption()`.
             *
             * If true, crypto will be automatically set up.
             *
             * Note that the appservice bot account is considered an intent.
             */
            encryption?: boolean;
        };
    }
    /**
     * Represents an application service. This provides helper utilities such as tracking
     * of user intents (clients that are aware of their membership in rooms).
     * @category Application services
     */
    export class Appservice extends EventEmitter {
        private options;
        /**
         * The metrics instance for this appservice. This will raise all metrics
         * from this appservice instance as well as any intents/MatrixClients created
         * by the appservice.
         */
        readonly metrics: Metrics;
        private readonly userPrefix;
        private readonly aliasPrefix;
        private readonly registration;
        private readonly storage;
        private readonly cryptoStorage;
        private readonly bridgeInstance;
        private app;
        private appServer;
        private intentsCache;
        private eventProcessors;
        private pendingTransactions;
        /**
         * Creates a new application service.
         * @param {IAppserviceOptions} options The options for the application service.
         */
        constructor(options: IAppserviceOptions);
        /**
         * Gets the express app instance which is serving requests. Not recommended for
         * general usage, but may be used to append routes to the web server.
         */
        get expressAppInstance(): import("express-serve-static-core").Express;
        /**
         * Gets the bridge-specific APIs for this application service.
         */
        get bridge(): MatrixBridge;
        /**
         * Get the application service's "bot" user ID (the sender_localpart).
         */
        get botUserId(): string;
        /**
         * Get the application service's "bot" Intent (the sender_localpart).
         * @returns {Intent} The intent for the application service itself.
         */
        get botIntent(): Intent;
        /**
         * Get the application service's "bot" MatrixClient (the sender_localpart).
         * Normally the botIntent should be used to ensure that the bot user is safely
         * handled.
         * @returns {MatrixClient} The client for the application service itself.
         */
        get botClient(): MatrixClient;
        /**
         * Starts the application service, opening the bind address to begin processing requests.
         * @returns {Promise<void>} resolves when started
         */
        begin(): Promise<void>;
        /**
         * Stops the application service, freeing the web server.
         */
        stop(): void;
        /**
         * Gets an intent for a given localpart. The user ID will be formed with the domain name given
         * in the constructor.
         * @param localpart The localpart to get an Intent for.
         * @returns {Intent} An Intent for the user.
         */
        getIntent(localpart: string): Intent;
        /**
         * Gets a full user ID for a given localpart. The user ID will be formed with the domain name given
         * in the constructor.
         * @param localpart The localpart to get a user ID for.
         * @returns {string} The user's ID.
         */
        getUserId(localpart: string): string;
        /**
         * Gets an Intent for a given user suffix. The prefix is automatically detected from the registration
         * options.
         * @param suffix The user's suffix
         * @returns {Intent} An Intent for the user.
         */
        getIntentForSuffix(suffix: string): Intent;
        /**
         * Gets a full user ID for a given suffix. The prefix is automatically detected from the registration
         * options.
         * @param suffix The user's suffix
         * @returns {string} The user's ID.
         */
        getUserIdForSuffix(suffix: string): string;
        /**
         * Gets an Intent for a given user ID.
         * @param {string} userId The user ID to get an Intent for.
         * @returns {Intent} An Intent for the user.
         */
        getIntentForUserId(userId: string): Intent;
        /**
         * Gets the suffix for the provided user ID. If the user ID is not a namespaced
         * user, this will return a falsey value.
         * @param {string} userId The user ID to parse
         * @returns {string} The suffix from the user ID.
         */
        getSuffixForUserId(userId: string): string;
        /**
         * Determines if a given user ID is namespaced by this application service.
         * @param {string} userId The user ID to check
         * @returns {boolean} true if the user is namespaced, false otherwise
         */
        isNamespacedUser(userId: string): boolean;
        /**
         * Gets a full alias for a given localpart. The alias will be formed with the domain name given
         * in the constructor.
         * @param localpart The localpart to get an alias for.
         * @returns {string} The alias.
         */
        getAlias(localpart: string): string;
        /**
         * Gets a full alias for a given suffix. The prefix is automatically detected from the registration
         * options.
         * @param suffix The alias's suffix
         * @returns {string} The alias.
         */
        getAliasForSuffix(suffix: string): string;
        /**
         * Gets the localpart of an alias for a given suffix. The prefix is automatically detected from the registration
         * options. Useful for the createRoom endpoint.
         * @param suffix The alias's suffix
         * @returns {string} The alias localpart.
         */
        getAliasLocalpartForSuffix(suffix: string): string;
        /**
         * Gets the suffix for the provided alias. If the alias is not a namespaced
         * alias, this will return a falsey value.
         * @param {string} alias The alias to parse
         * @returns {string} The suffix from the alias.
         */
        getSuffixForAlias(alias: string): string;
        /**
         * Determines if a given alias is namespaced by this application service.
         * @param {string} alias The alias to check
         * @returns {boolean} true if the alias is namespaced, false otherwise
         */
        isNamespacedAlias(alias: string): boolean;
        /**
         * Adds a preprocessor to the event pipeline. When this appservice encounters an event, it
         * will try to run it through the preprocessors it can in the order they were added.
         * @param {IPreprocessor} preprocessor the preprocessor to add
         */
        addPreprocessor(preprocessor: IPreprocessor): void;
        /**
         * Sets the visibility of a room in the appservice's room directory.
         * @param {string} networkId The network ID to group the room under.
         * @param {string} roomId The room ID to manipulate the visibility of.
         * @param {"public" | "private"} visibility The visibility to set for the room.
         * @return {Promise<any>} resolves when the visibility has been updated.
         */
        setRoomDirectoryVisibility(networkId: string, roomId: string, visibility: "public" | "private"): Promise<any>;
        private processEphemeralEvent;
        private processEvent;
        private processMembershipEvent;
        private isAuthed;
        private onTransaction;
        private onUser;
        private onRoomAlias;
        private onThirdpartyProtocol;
        private handleThirdpartyObject;
        private onThirdpartyUser;
        private onThirdpartyLocation;
    }
}
declare module "e2ee/RoomTracker" {
    import { MatrixClient } from "MatrixClient";
    import { EncryptionEventContent } from "models/events/EncryptionEvent";
    /**
     * Tracks room encryption status for a MatrixClient.
     * @category Encryption
     */
    export class RoomTracker {
        private client;
        constructor(client: MatrixClient);
        /**
         * Prepares the room tracker to track the given rooms.
         * @param {string[]} roomIds The room IDs to track. This should be the joined rooms set.
         */
        prepare(roomIds: string[]): Promise<void>;
        /**
         * Queues a room check for the tracker. If the room needs an update to the store, an
         * update will be made.
         * @param {string} roomId The room ID to check.
         */
        queueRoomCheck(roomId: string): Promise<void>;
        /**
         * Gets the room's crypto configuration, as known by the underlying store. If the room is
         * not encrypted then this will return an empty object.
         * @param {string} roomId The room ID to get the config for.
         * @returns {Promise<Partial<EncryptionEventContent>>} Resolves to the encryption config.
         */
        getRoomCryptoConfig(roomId: string): Promise<Partial<EncryptionEventContent>>;
    }
}
declare module "e2ee/decorators" {
    /**
     * Flags a MatrixClient function as needing end-to-end encryption enabled.
     * @category Encryption
     */
    export function requiresCrypto(): (target: any, propertyKey: string, descriptor: PropertyDescriptor) => void;
    /**
     * Flags a CryptoClient function as needing the CryptoClient to be ready.
     * @category Encryption
     */
    export function requiresReady(): (target: any, propertyKey: string, descriptor: PropertyDescriptor) => void;
}
declare module "e2ee/DeviceTracker" {
    import { MatrixClient } from "MatrixClient";
    import { UserDevice } from "models/Crypto";
    /**
     * Tracks user devices for encryption operations.
     * @category Encryption
     */
    export class DeviceTracker {
        private client;
        private deviceListUpdates;
        constructor(client: MatrixClient);
        /**
         * Gets the device lists for the given user IDs. Outdated device lists will be updated before
         * returning.
         * @param {string[]} userIds The user IDs to get the device lists of.
         * @returns {Promise<Record<string, UserDevice[]>>} Resolves to a map of user ID to device list.
         * If a user has no devices, they may be excluded from the result or appear as an empty array.
         */
        getDevicesFor(userIds: string[]): Promise<Record<string, UserDevice[]>>;
        /**
         * Flags multiple user's device lists as outdated, optionally queuing an immediate update.
         * @param {string} userIds The user IDs to flag the device lists of.
         * @param {boolean} resync True (default) to queue an immediate update, false otherwise.
         * @returns {Promise<void>} Resolves when the flagging has completed. Will wait for the resync
         * if requested too.
         */
        flagUsersOutdated(userIds: string[], resync?: boolean): Promise<void>;
        /**
         * Updates multiple user's device lists regardless of outdated flag.
         * @param {string[]} userIds The user IDs to update.
         * @returns {Promise<void>} Resolves when complete.
         */
        updateUsersDeviceLists(userIds: string[]): Promise<void>;
    }
}
declare module "models/events/EncryptedRoomEvent" {
    import { RoomEvent } from "models/events/RoomEvent";
    import { EncryptionAlgorithm, IMegolmEncrypted } from "models/Crypto";
    /**
     * The content definition for m.room.encrypted events
     * @category Matrix event contents
     * @see EncryptedRoomEvent
     */
    export interface EncryptedRoomEventContent {
        algorithm: EncryptionAlgorithm;
        /**
         * For m.megolm.v1.aes-sha2 messages. The sender's Curve25519 key.
         */
        sender_key?: string;
        /**
         * For m.megolm.v1.aes-sha2 messages. The session ID established by the sender.
         */
        session_id?: string;
        /**
         * For m.megolm.v1.aes-sha2 messages. The encrypted payload.
         */
        ciphertext?: string;
        /**
         * For m.megolm.v1.aes-sha2 messages. The sender's device ID.
         */
        device_id?: string;
    }
    /**
     * Represents an m.room.encrypted room event
     * @category Matrix events
     */
    export class EncryptedRoomEvent extends RoomEvent<EncryptedRoomEventContent> {
        constructor(event: any);
        /**
         * The encryption algorithm used on the event. Should match the m.room.encryption
         * state config.
         */
        get algorithm(): EncryptionAlgorithm;
        /**
         * The Megolm encrypted payload information.
         */
        get megolmProperties(): IMegolmEncrypted;
    }
}
declare module "models/events/InvalidEventError" {
    /**
     * Thrown when an event is invalid.
     * @category Matrix events
     */
    export class InvalidEventError extends Error {
        constructor(message?: string);
    }
    /**
     * Thrown when an event is redacted.
     * @category Matrix events
     */
    export class EventRedactedError extends InvalidEventError {
        constructor(message?: string);
    }
}
declare module "models/events/MessageEvent" {
    import { RoomEvent } from "models/events/RoomEvent";
    /**
     * The types of messages that are valid in Matrix.
     * @category Matrix event info
     * @see MessageEventContent
     */
    export type MessageType = "m.text" | "m.emote" | "m.notice" | "m.image" | "m.file" | "m.audio" | "m.location" | "m.video" | string;
    /**
     * Information about a file in Matrix
     * @category Matrix event info
     * @see MessageEventContent
     */
    export interface FileInfo {
        /**
         * The size of the file in bytes.
         */
        size?: number;
        /**
         * The type of file.
         */
        mimetype?: string;
    }
    /**
     * Information about a thumbnail in Matrix
     * @category Matrix event info
     * @see MessageEventContent
     */
    export interface ThumbnailInfo {
        /**
         * The size of the thumbnail in bytes.
         */
        size?: number;
        /**
         * The type of thumbnail.
         */
        mimetype?: string;
        /**
         * The intended height of the thumbnail in pixels.
         */
        h: number;
        /**
         * The intended width of the thumbnail in pixels.
         */
        w: number;
    }
    /**
     * Information about a file's thumbnail.
     * @category Matrix event info
     * @see MessageEventContent
     */
    export interface ThumbnailedFileInfo {
        /**
         * A URL to a thumbnail for the file, if unencrypted.
         */
        thumbnail_url?: string;
        /**
         * The encrypted thumbnail file information, if encrypted.
         */
        thumbnail_file?: EncryptedFile;
        /**
         * Information about the thumbnail. Optionally included if a thumbnail_url is specified.
         */
        thumbnail_info?: ThumbnailInfo;
    }
    /**
     * Information about a file that has a thumbnail
     * @category Matrix event info
     * @see MessageEventContent
     */
    export interface FileWithThumbnailInfo extends FileInfo, ThumbnailedFileInfo {
    }
    /**
     * Information about a file that has a width and height.
     * @category Matrix event info
     * @see MessageEventContent
     */
    export interface DimensionalFileInfo extends FileWithThumbnailInfo {
        /**
         * The intended height of the media in pixels.
         */
        h: number;
        /**
         * The intended width of the media in pixels.
         */
        w: number;
    }
    /**
     * Information about a file that has a time dimension.
     * @category Matrix event info
     * @see MessageEventContent
     */
    export interface TimedFileInfo extends FileInfo {
        /**
         * The duration of the media in milliseconds.
         */
        duration: number;
    }
    /**
     * Information about a video file.
     * @category Matrix event info
     * @see MessageEventContent
     */
    export interface VideoFileInfo extends DimensionalFileInfo, TimedFileInfo {
    }
    /**
     * The content definition for m.room.message events with a type of m.audio
     * @category Matrix event contents
     * @see MessageEvent
     */
    export interface AudioMessageEventContent extends FileMessageEventContent {
        /**
         * Information about the file.
         */
        info?: TimedFileInfo;
    }
    /**
     * The content definition for m.room.message events with a type of m.video
     * @category Matrix event contents
     * @see MessageEvent
     */
    export interface VideoMessageEventContent extends FileMessageEventContent {
        /**
         * Information about the file.
         */
        info?: VideoFileInfo;
    }
    /**
     * The content definition for m.room.message events with a type of m.image
     * @category Matrix event contents
     * @see MessageEvent
     */
    export interface ImageMessageEventContent extends FileMessageEventContent {
        /**
         * Information about the file.
         */
        info?: DimensionalFileInfo;
    }
    /**
     * The content definition for m.room.message events with a type of m.file
     * @category Matrix event contents
     * @see MessageEvent
     */
    export interface FileMessageEventContent extends MessageEventContent {
        /**
         * Information about the file.
         */
        info?: FileWithThumbnailInfo;
        /**
         * URL to the file, if unencrypted.
         */
        url: string;
        /**
         * The encrypted file, if encrypted.
         */
        file: EncryptedFile;
    }
    /**
     * An encrypted file.
     * @category Matrix event contents
     * @see MessageEvent
     */
    export interface EncryptedFile {
        url: string;
        key: {
            kty: "oct";
            key_ops: string[];
            alg: "A256CTR";
            k: string;
            ext: true;
        };
        iv: string;
        hashes: {
            sha256: string;
        };
        v: "v2";
    }
    /**
     * The content definition for m.room.message events with a type of m.location
     * @category Matrix event contents
     * @see MessageEvent
     */
    export interface LocationMessageEventContent extends MessageEventContent {
        /**
         * Information about the location.
         */
        info?: ThumbnailedFileInfo;
        /**
         * A geo URI of the location.
         */
        geo_uri?: string;
    }
    /**
     * The content definition for m.room.message events with types of m.text, m.emote, and m.notice
     * @category Matrix event contents
     * @see MessageEvent
     */
    export interface TextualMessageEventContent extends MessageEventContent {
        format?: string;
        formatted_body?: string;
    }
    /**
     * The content definition for m.room.message events
     * @category Matrix event contents
     * @see MessageEvent
     */
    export interface MessageEventContent {
        body: string;
        msgtype: MessageType;
        external_url?: string;
    }
    /**
     * Represents an m.room.message room event
     * @category Matrix events
     */
    export class MessageEvent<T extends MessageEventContent> extends RoomEvent<T> {
        constructor(event: any);
        /**
         * Whether or not the event is redacted (or looked redacted).
         */
        get isRedacted(): boolean;
        /**
         * The message's type.
         */
        get messageType(): MessageType;
        /**
         * The `body` of the message.
         */
        get textBody(): string;
        /**
         * The `external_url` of the message, if it exists
         */
        get externalUrl(): string | undefined;
    }
}
declare module "b64" {
    /**
     * Encodes Base64.
     * @category Utilities
     * @param {ArrayBuffer | Uint8Array} b The buffer to encode.
     * @returns {string} The Base64 string.
     */
    export function encodeBase64(b: ArrayBuffer | Uint8Array): string;
    /**
     * Encodes Unpadded Base64.
     * @category Utilities
     * @param {ArrayBuffer | Uint8Array} b The buffer to encode.
     * @returns {string} The Base64 string.
     */
    export function encodeUnpaddedBase64(b: ArrayBuffer | Uint8Array): string;
    /**
     * Encodes URL-Safe Unpadded Base64.
     * @category Utilities
     * @param {ArrayBuffer | Uint8Array} b The buffer to encode.
     * @returns {string} The Base64 string.
     */
    export function encodeUnpaddedUrlSafeBase64(b: ArrayBuffer | Uint8Array): string;
    /**
     * Decodes Base64.
     * @category Utilities
     * @param {string} s The Base64 string.
     * @returns {Uint8Array} The encoded data as a buffer.
     */
    export function decodeBase64(s: string): Uint8Array;
    /**
     * Decodes Unpadded Base64.
     * @category Utilities
     * @param {string} s The Base64 string.
     * @returns {Uint8Array} The encoded data as a buffer.
     */
    export function decodeUnpaddedBase64(s: string): Uint8Array;
    /**
     * Decodes URL-Safe Unpadded Base64.
     * @category Utilities
     * @param {string} s The Base64 string.
     * @returns {Uint8Array} The encoded data as a buffer.
     */
    export function decodeUnpaddedUrlSafeBase64(s: string): Uint8Array;
}
declare module "e2ee/CryptoClient" {
    import { MatrixClient } from "MatrixClient";
    import { IMegolmEncrypted, IOlmEncrypted, IOlmSession, IToDeviceMessage, OTKCounts, Signatures } from "models/Crypto";
    import { EncryptedRoomEvent } from "models/events/EncryptedRoomEvent";
    import { RoomEvent } from "models/events/RoomEvent";
    import { EncryptedFile } from "models/events/MessageEvent";
    /**
     * Manages encryption for a MatrixClient. Get an instance from a MatrixClient directly
     * rather than creating one manually.
     * @category Encryption
     */
    export class CryptoClient {
        private client;
        private ready;
        private deviceId;
        private pickleKey;
        private deviceEd25519;
        private deviceCurve25519;
        private maxOTKs;
        private roomTracker;
        private deviceTracker;
        constructor(client: MatrixClient);
        /**
         * The device ID for the MatrixClient.
         */
        get clientDeviceId(): string;
        /**
         * Whether or not the crypto client is ready to be used. If not ready, prepare() should be called.
         * @see prepare
         */
        get isReady(): boolean;
        private getOlmAccount;
        private storeAndFreeOlmAccount;
        /**
         * Prepares the crypto client for usage.
         * @param {string[]} roomIds The room IDs the MatrixClient is joined to.
         */
        prepare(roomIds: string[]): Promise<void>;
        /**
         * Checks if a room is encrypted.
         * @param {string} roomId The room ID to check.
         * @returns {Promise<boolean>} Resolves to true if encrypted, false otherwise.
         */
        isRoomEncrypted(roomId: string): Promise<boolean>;
        /**
         * Updates the One Time Key counts, potentially triggering an async upload of more
         * one time keys.
         * @param {OTKCounts} counts The current counts to work within.
         * @returns {Promise<void>} Resolves when complete.
         */
        updateCounts(counts: OTKCounts): Promise<void>;
        /**
         * Updates the client's fallback key.
         * @returns {Promise<void>} Resolves when complete.
         */
        updateFallbackKey(): Promise<void>;
        /**
         * Signs an object using the device keys.
         * @param {object} obj The object to sign.
         * @returns {Promise<Signatures>} The signatures for the object.
         */
        sign(obj: object): Promise<Signatures>;
        /**
         * Verifies a signature on an object.
         * @param {object} obj The signed object.
         * @param {string} key The key which has supposedly signed the object.
         * @param {string} signature The advertised signature.
         * @returns {Promise<boolean>} Resolves to true if a valid signature, false otherwise.
         */
        verifySignature(obj: object, key: string, signature: string): Promise<boolean>;
        /**
         * Flags multiple user's device lists as outdated, optionally queuing an immediate update.
         * @param {string} userIds The user IDs to flag the device lists of.
         * @param {boolean} resync True (default) to queue an immediate update, false otherwise.
         * @returns {Promise<void>} Resolves when the device lists have been flagged. Will also wait
         * for the resync if one was requested.
         */
        flagUsersDeviceListsOutdated(userIds: string[], resync?: boolean): Promise<void>;
        /**
         * Gets or creates Olm sessions for the given users and devices. Where sessions cannot be created,
         * the user/device will be excluded from the returned map.
         * @param {Record<string, string[]>} userDeviceMap Map of user IDs to device IDs
         * @param {boolean} force If true, force creation of a session for the referenced users.
         * @returns {Promise<Record<string, Record<string, IOlmSession>>>} Resolves to a map of user ID to device
         * ID to session. Users/devices which cannot have sessions made will not be included, thus the object
         * may be empty.
         */
        getOrCreateOlmSessions(userDeviceMap: Record<string, string[]>, force?: boolean): Promise<Record<string, Record<string, IOlmSession>>>;
        private encryptAndSendOlmMessage;
        /**
         * Encrypts the details of a room event, returning an encrypted payload to be sent in an
         * `m.room.encrypted` event to the room. If needed, this function will send decryption keys
         * to the appropriate devices in the room (this happens when the Megolm session rotates or
         * gets created).
         * @param {string} roomId The room ID to encrypt within. If the room is not encrypted, an
         * error is thrown.
         * @param {string} eventType The event type being encrypted.
         * @param {any} content The event content being encrypted.
         * @returns {Promise<IMegolmEncrypted>} Resolves to the encrypted content for an `m.room.encrypted` event.
         */
        encryptRoomEvent(roomId: string, eventType: string, content: any): Promise<IMegolmEncrypted>;
        /**
         * Decrypts a room event. Currently only supports Megolm-encrypted events (default for this SDK).
         * @param {EncryptedRoomEvent} event The encrypted event.
         * @param {string} roomId The room ID where the event was sent.
         * @returns {Promise<RoomEvent<unknown>>} Resolves to a decrypted room event, or rejects/throws with
         * an error if the event is undecryptable.
         */
        decryptRoomEvent(event: EncryptedRoomEvent, roomId: string): Promise<RoomEvent<unknown>>;
        /**
         * Handles an inbound to-device message, decrypting it if needed. This will not throw
         * under normal circumstances and should always resolve successfully.
         * @param {IToDeviceMessage<IOlmEncrypted>} message The message to process.
         * @returns {Promise<void>} Resolves when complete. Should never fail.
         */
        processInboundDeviceMessage(message: IToDeviceMessage<IOlmEncrypted>): Promise<void>;
        private handleInboundRoomKey;
        private storeInboundGroupSession;
        private establishNewOlmSession;
        /**
         * Encrypts a file for uploading in a room, returning the encrypted data and information
         * to include in a message event (except media URL) for sending.
         * @param {Buffer} file The file to encrypt.
         * @returns {{buffer: Buffer, file: Omit<EncryptedFile, "url">}} Resolves to the encrypted
         * contents and file information.
         */
        encryptMedia(file: Buffer): Promise<{
            buffer: Buffer;
            file: Omit<EncryptedFile, "url">;
        }>;
        /**
         * Decrypts a previously-uploaded encrypted file, validating the fields along the way.
         * @param {EncryptedFile} file The file to decrypt.
         * @returns {Promise<Buffer>} Resolves to the decrypted file contents.
         */
        decryptMedia(file: EncryptedFile): Promise<Buffer>;
    }
}
declare module "helpers/RichReply" {
    /**
     * Helper for creating rich replies.
     * @category Utilities
     */
    export class RichReply {
        private constructor();
        /**
         * Generates the event content required to reply to the provided event with the
         * provided text.
         * @param {string} roomId the room ID the event being replied to resides in
         * @param {any} event the event to reply to
         * @param {string} withText the plain text to reply with
         * @param {string} withHtml the HTML to reply with
         * @returns {any} the content of the event representing the reply
         */
        static createFor(roomId: string, event: any, withText: string, withHtml: string): any;
    }
}
declare module "helpers/Permalinks" {
    /**
     * The parts of a permalink.
     * @see Permalinks
     * @category Utilities
     */
    export interface PermalinkParts {
        /**
         * The room ID or alias the permalink references. May be null.
         */
        roomIdOrAlias: string;
        /**
         * The user ID the permalink references. May be null.
         */
        userId: string;
        /**
         * The event ID the permalink references. May be null.
         */
        eventId: string;
        /**
         * The servers the permalink is routed through. May be null or empty.
         */
        viaServers: string[];
    }
    /**
     * Functions for handling permalinks
     * @category Utilities
     */
    export class Permalinks {
        private constructor();
        private static encodeViaArgs;
        /**
         * Creates a room permalink.
         * @param {string} roomIdOrAlias The room ID or alias to create a permalink for.
         * @param {string[]} viaServers The servers to route the permalink through.
         * @returns {string} A room permalink.
         */
        static forRoom(roomIdOrAlias: string, viaServers?: string[]): string;
        /**
         * Creates a user permalink.
         * @param {string} userId The user ID to create a permalink for.
         * @returns {string} A user permalink.
         */
        static forUser(userId: string): string;
        /**
         * Creates an event permalink.
         * @param {string} roomIdOrAlias The room ID or alias to create a permalink in.
         * @param {string} eventId The event ID to reference in the permalink.
         * @param {string[]} viaServers The servers to route the permalink through.
         * @returns {string} An event permalink.
         */
        static forEvent(roomIdOrAlias: string, eventId: string, viaServers?: string[]): string;
        /**
         * Parses a permalink URL into usable parts.
         * @param {string} matrixTo The matrix.to URL to parse.
         * @returns {PermalinkParts} The parts of the permalink.
         */
        static parseUrl(matrixTo: string): PermalinkParts;
    }
}
declare module "helpers/MentionPill" {
    import { MatrixClient } from "MatrixClient";
    /**
     * Represents a system for generating a mention pill for an entity.
     * @category Utilities
     */
    export class MentionPill {
        private entityPermalink;
        private displayName;
        private constructor();
        /**
         * The HTML component of the mention.
         */
        get html(): string;
        /**
         * The plain text component of the mention.
         */
        get text(): string;
        /**
         * Creates a new mention for a user in an optional room.
         * @param {string} userId The user ID the mention is for.
         * @param {String} inRoomId Optional room ID the user is being mentioned in, for the aesthetics of the mention.
         * @param {MatrixClient} client Optional client for creating a more pleasing mention.
         * @returns {Promise<MentionPill>} Resolves to the user's mention.
         */
        static forUser(userId: string, inRoomId?: string, client?: MatrixClient): Promise<MentionPill>;
        /**
         * Creates a new mention for a room (not @room, but the room itself to be linked).
         * @param {string} roomIdOrAlias The room ID or alias to mention.
         * @param {MatrixClient} client Optional client for creating a more pleasing mention.
         * @returns {Promise<MentionPill>} Resolves to the room's mention.
         */
        static forRoom(roomIdOrAlias: string, client?: MatrixClient): Promise<MentionPill>;
        /**
         * Creates a mention from static information.
         * @param {string} userId The user ID the mention is for.
         * @param {string} displayName The user's display name.
         * @returns {MentionPill} The mention for the user.
         */
        static withDisplayName(userId: string, displayName: string): MentionPill;
    }
}
declare module "helpers/MatrixGlob" {
    /**
     * Represents a common Matrix glob. This is commonly used
     * for server ACLs and similar functions.
     * @category Utilities
     */
    export class MatrixGlob {
        /**
         * The regular expression which represents this glob.
         */
        readonly regex: RegExp;
        /**
         * Creates a new Matrix Glob
         * @param {string} glob The glob to convert. Eg: "*.example.org"
         */
        constructor(glob: string);
        /**
         * Tests the glob against a value, returning true if it matches.
         * @param {string} val The value to test.
         * @returns {boolean} True if the value matches the glob, false otherwise.
         */
        test(val: string): boolean;
    }
}
declare module "models/events/MembershipEvent" {
    import { StateEvent } from "models/events/RoomEvent";
    /**
     * The types of membership that are valid in Matrix.
     * @category Matrix event info
     * @see MembershipEventContent
     */
    export type Membership = "join" | "leave" | "ban" | "invite";
    /**
     * The effective membership states a user can be in.
     * @category Matrix event info
     * @see MembershipEventContent
     */
    export type EffectiveMembership = "join" | "leave" | "invite";
    /**
     * The content definition for m.room.member events
     * @category Matrix event contents
     * @see MembershipEvent
     */
    export interface MembershipEventContent {
        avatar_url?: string;
        displayname?: string;
        membership: Membership;
        is_direct?: boolean;
        unsigned?: any;
        third_party_invite?: {
            display_name: string;
            signed: any;
        };
    }
    /**
     * Represents an m.room.member state event
     * @category Matrix events
     */
    export class MembershipEvent extends StateEvent<MembershipEventContent> {
        constructor(event: any);
        /**
         * True if the membership event targets the sender. False otherwise.
         *
         * This will typically by false for kicks and bans.
         */
        get ownMembership(): boolean;
        /**
         * The user ID the membership affects.
         */
        get membershipFor(): string;
        /**
         * The user's membership.
         */
        get membership(): Membership;
        /**
         * The user's effective membership.
         */
        get effectiveMembership(): EffectiveMembership;
    }
}
declare module "helpers/ProfileCache" {
    import { MatrixClient, MatrixProfile } from "index";
    import { Appservice } from "appservice/Appservice";
    /**
     * Functions for avoiding calls to profile endpoints. Useful for bots when
     * people are mentioned often or bridges which need profile information
     * often.
     * @category Utilities
     */
    export class ProfileCache {
        private client;
        private cache;
        /**
         * Creates a new profile cache.
         * @param {number} maxEntries The maximum number of entries to cache.
         * @param {number} maxAgeMs The maximum age of an entry in milliseconds.
         * @param {MatrixClient} client The client to use to get profile updates.
         */
        constructor(maxEntries: number, maxAgeMs: number, client: MatrixClient);
        private getCacheKey;
        /**
         * Watch for profile changes to cached entries with the provided client. The
         * same client will also be used to update the user's profile in the cache.
         * @param {MatrixClient} client The client to watch for profile changes with.
         */
        watchWithClient(client: MatrixClient): void;
        /**
         * Watch for profile changes to cached entries with the provided application
         * service. The clientFn will be called to get the relevant client for any
         * updates. If the clientFn is null, the appservice's bot user will be used.
         * The clientFn takes two arguments: the user ID being updated and the room ID
         * they are being updated in (shouldn't be null). The return value should be the
         * MatrixClient to use, or null to use the appservice's bot client. The same
         * client will be used to update the user's general profile, if that profile
         * is cached.
         * @param {Appservice} appservice The application service to watch for profile changes with.
         * @param {Function} clientFn The function to use to acquire profile updates with. If null, the appservice's bot client will be used.
         */
        watchWithAppservice(appservice: Appservice, clientFn?: (userId: string, roomId: string) => MatrixClient): void;
        /**
         * Gets a profile for a user in optional room.
         * @param {string} userId The user ID to get a profile for.
         * @param {string|null} roomId Optional room ID to get a per-room profile for the user.
         * @returns {Promise<MatrixProfile>} Resolves to the user's profile.
         */
        getUserProfile(userId: string, roomId?: string): Promise<MatrixProfile>;
        private getUserProfileWith;
        private tryUpdateProfile;
    }
}
declare module "helpers/MatrixEntity" {
    /**
     * Represents a Matrix entity
     * @category Utilities
     */
    export class MatrixEntity {
        private fullId;
        private entityLocalpart;
        private entityDomain;
        /**
         * Creates a new Matrix entity
         * @param {string} fullId The full ID of the entity
         */
        constructor(fullId: string);
        /**
         * The localpart for the entity
         */
        get localpart(): string;
        /**
         * The domain for the entity
         */
        get domain(): string;
        toString(): string;
    }
    /**
     * Represents a Matrix user ID
     * @category Utilities
     */
    export class UserID extends MatrixEntity {
        constructor(userId: string);
    }
    /**
     * Represents a Matrix room alias
     * @category Utilities
     */
    export class RoomAlias extends MatrixEntity {
        constructor(alias: string);
    }
}
declare module "logging/RichConsoleLogger" {
    import { ILogger } from "logging/ILogger";
    import * as chalk from "chalk";
    /**
     * Prints to the console with colors and a format.
     * @category Logging
     */
    export class RichConsoleLogger implements ILogger {
        protected chalkDebug: chalk.Chalk;
        protected chalkInfo: chalk.Chalk;
        protected chalkWarning: chalk.Chalk;
        protected chalkError: chalk.Chalk;
        protected chalkTimestamp: chalk.Chalk;
        protected chalkModule: chalk.Chalk;
        protected getTimestamp(): string;
        trace(module: string, ...messageOrObject: any[]): void;
        debug(module: string, ...messageOrObject: any[]): void;
        error(module: string, ...messageOrObject: any[]): void;
        info(module: string, ...messageOrObject: any[]): void;
        warn(module: string, ...messageOrObject: any[]): void;
    }
}
declare module "mixins/AutojoinRoomsMixin" {
    import { MatrixClient } from "MatrixClient";
    import { Appservice } from "appservice/Appservice";
    /**
     * Automatically accepts invites for rooms
     * @category Mixins
     */
    export class AutojoinRoomsMixin {
        static setupOnClient(client: MatrixClient): void;
        static setupOnAppservice(appservice: Appservice, conditional?: (inviteEvent: any) => boolean): void;
    }
}
declare module "mixins/AutojoinUpgradedRoomsMixin" {
    import { MatrixClient } from "MatrixClient";
    import { Appservice } from "appservice/Appservice";
    /**
     * Automatically tries to join upgraded rooms
     * @category Mixins
     */
    export class AutojoinUpgradedRoomsMixin {
        static setupOnClient(client: MatrixClient): void;
        static setupOnAppservice(appservice: Appservice): void;
    }
}
declare module "models/events/PresenceEvent" {
    import { MatrixEvent } from "models/events/Event";
    /**
     * The allowed states of presence in Matrix
     * @category Matrix event info
     * @see PresenceEventContent
     */
    export type PresenceState = "online" | "offline" | "unavailable";
    /**
     * Event content for m.presence events
     * @category Matrix event contents
     * @see PresenceEvent
     */
    export interface PresenceEventContent {
        /**
         * The avatar URL for the user, if any.
         */
        avatar_url?: string;
        /**
         * The display name for the user, if any.
         */
        displayname?: string;
        /**
         * How long ago the user performed some action, in milliseconds.
         */
        last_active_ago?: number;
        /**
         * The user's presence state.
         */
        presence: PresenceState;
        /**
         * Whether or not the user is currently active.
         */
        currently_active?: boolean;
        /**
         * A status message associated with this presence.
         */
        status_msg?: string;
    }
    /**
     * Wraps a m.presence ephemeral event in Matrix
     * @category Matrix events
     */
    export class PresenceEvent extends MatrixEvent<PresenceEventContent> {
        constructor(event: any);
        /**
         * The current presence state for the user.
         */
        get presence(): PresenceState;
    }
}
declare module "models/Presence" {
    import { PresenceEventContent, PresenceState } from "models/events/PresenceEvent";
    /**
     * Presence information for a user.
     * @category Models
     */
    export class Presence {
        protected presence: PresenceEventContent;
        constructor(presence: PresenceEventContent);
        /**
         * The state for this presence update.
         */
        get state(): PresenceState;
        /**
         * The status message which accompanies this presence. May be falsey.
         */
        get statusMessage(): string;
        /**
         * How long ago in milliseconds this presence was changed. May be falsey.
         */
        get lastActiveAgo(): number;
        /**
         * Whether or not the user is currently active.
         */
        get currentlyActive(): boolean;
    }
}
declare module "models/EventContext" {
    import { RoomEvent, RoomEventContent, StateEvent } from "models/events/RoomEvent";
    export interface EventContext {
        /**
         * The event that was used to build this context.
         */
        event: RoomEvent<RoomEventContent>;
        /**
         * The events that happened before the contextual event.
         */
        before: RoomEvent<RoomEventContent>[];
        /**
         * The events that happened after the contextual event.
         */
        after: RoomEvent<RoomEventContent>[];
        /**
         * The state of the room at the point of the last event returned.
         */
        state: StateEvent<RoomEventContent>[];
    }
}
declare module "models/PowerLevelBounds" {
    /**
     * Information on the bounds of a power level change a user can apply.
     */
    export interface PowerLevelBounds {
        /**
         * Whether or not the user can even modify the power level of the user. This
         * will be false if the user can't send power level events, or the user is
         * unobtainably high in power.
         */
        canModify: boolean;
        /**
         * The maximum possible power level the user can set on the target user.
         */
        maximumPossibleLevel: number;
    }
}
declare module "simple-validation" {
    /**
     * Validate the 'order' parameter of a child space entry. It must be a
     * string between the range of \x20 - \x7F and contain no more than 50
     * characters.
     * @param {string} order The 'order' parameter of a m.space.child
     * @throws {Error} If the string is not valid
     * @returns {boolean} True if the string is valid
     */
    export function validateSpaceOrderString(order: string): true;
}
declare module "models/events/SpaceChildEvent" {
    import { StateEvent } from "models/events/RoomEvent";
    /**
     * The content definition for m.space.child events
     * @category Matrix event contents
     * @see SpaceChildEvent
     */
    export interface SpaceChildEventContent {
        /**
         * The servers the client should provide to the server when trying to join the room.
         * When not provided or empty, the child is considered not part of the space.
         */
        via: string[];
        /**
         * A short string to differentiate the rendering order of entities.
         * @see validateSpaceOrderString
         */
        order?: string;
        /**
         * Whether or not the entity is intended to be a suggested entity.
         */
        suggested?: boolean;
    }
    /**
     * Represents an m.space.child state event
     * @category Matrix events
     */
    export class SpaceChildEvent extends StateEvent<SpaceChildEventContent> {
        constructor(event: any);
        /**
         * The room ID of the space or room this child represents.
         */
        get entityRoomId(): string;
        /**
         * Whether or not this child is "active" or valid. Inactive children are
         * not considered part of a space.
         */
        get isActiveChild(): boolean;
        /**
         * The servers the client should provide to the homeserver when trying to
         * join the entity (room). May be empty or falsey to denote the child is
         * inactive.
         */
        get viaServers(): string[];
        /**
         * An optional short string to differentiate the rendering order of entities.
         * @see validateSpaceOrderString
         */
        get order(): string;
        /**
         * Whether or not the child is a suggested entity for users to join.
         */
        get suggested(): boolean;
    }
}
declare module "models/Spaces" {
    import { MatrixClient } from "MatrixClient";
    import { SpaceChildEvent } from "models/events/SpaceChildEvent";
    /**
     * Options to be specified when creating a Space.
     * @category Models
     */
    export interface SpaceCreateOptions {
        /**
         * The name of the space.
         */
        name: string;
        /**
         * The topic/description for the space.
         */
        topic?: string;
        /**
         * An MXC URI for the space's avatar.
         */
        avatarUrl?: string;
        /**
         * Whether or not the space should be publicly joinable or not.
         */
        isPublic: boolean;
        /**
         * Optional localpart for the alias of the space.
         */
        localpart?: string;
        /**
         * User IDs to invite to the space upon creation.
         */
        invites?: string[];
    }
    /**
     * Options for displaying/handling a child room/space.
     * @category Models
     */
    export interface SpaceChildEntityOptions {
        /**
         * Whether or not the entity is intended to be a suggested entity.
         */
        suggested?: boolean;
        /**
         * Servers to try and join through. When not provided, the SDK will try to
         * determine a set.
         */
        via?: string[];
        /**
         * A short string to differentiate the rendering order of entities.
         * @see validateSpaceOrderString
         */
        order?: string;
    }
    /**
     * Options for creating a new child space or room.
     * @category Models
     */
    export type NewChildOpts = SpaceCreateOptions & SpaceChildEntityOptions;
    /**
     * A mapping of room ID to space child information.
     * @category Models
     */
    export interface SpaceEntityMap {
        [roomId: string]: SpaceChildEvent;
    }
    /**
     * An instance representing a Matrix Space. A space is tied to a room.
     * @category Models
     */
    export class Space {
        readonly roomId: string;
        readonly client: MatrixClient;
        constructor(roomId: string, client: MatrixClient);
        /**
         * Creates a new child space under this space.
         * @param {SpaceCreateOptions} opts The options for the new space.
         * @returns {Promise<Space>} Resolves to the created space.
         */
        createChildSpace(opts: NewChildOpts): Promise<Space>;
        /**
         * Adds a child space to the space. Must be joined to both spaces.
         * @param {Space} space The space to add.
         * @param {SpaceChildEntityOptions} childOpts Related options for the child's representation.
         * @returns {Promise<Space>} Resolves when complete.
         */
        addChildSpace(space: Space, childOpts?: SpaceChildEntityOptions): Promise<void>;
        /**
         * Removes a child space from the space. Must be joined to the current space (not needed for child space).
         * @param {Space} space The space to remove.
         * @returns {Promise<void>} Resolves when complete.
         */
        removeChildSpace(space: Space): Promise<void>;
        /**
         * Adds a child room to the space. Must be joined to both the room and the space.
         * @param {string} roomId The room ID to add.
         * @param {SpaceChildEntityOptions} childOpts Additional options for the child space.
         * @returns {Promise<void>} Resolves when complete.
         */
        addChildRoom(roomId: string, childOpts?: SpaceChildEntityOptions): Promise<void>;
        /**
         * Removes a child room from the space. Must be joined to the current space (not needed for child room).
         * @param {string} roomId The room ID to remove.
         * @returns {Promise<void>} Resolves when complete.
         */
        removeChildRoom(roomId: string): Promise<void>;
        /**
         * Gets all the child rooms on the space. These may be spaces or other rooms.
         * @returns {Promise<SpaceEntityMap>} Resolves to a map of children for this space.
         */
        getChildEntities(): Promise<SpaceEntityMap>;
        /**
         * Invite a user to the current space.
         * @param {string} userId The user ID to invite.
         * @returns {Promise<void>} Resolves when completed.
         */
        inviteUser(userId: string): Promise<any>;
    }
}
declare module "models/Account" {
    /**
     * Response to a `/whoami` request.
     * @category Models
     */
    export interface IWhoAmI {
        user_id: string;
        device_id?: string;
    }
}
declare module "models/events/EventKind" {
    /**
     * Represents the different kinds of events a bot/appservice might see.
     * @category Matrix events
     */
    export enum EventKind {
        /**
         * A room event. This could be a message event or a state event, and is associated with
         * a room.
         */
        RoomEvent = "room",
        /**
         * An ephemeral event, such as typing notifications or presence.
         */
        EphemeralEvent = "ephemeral"
    }
}
declare module "models/events/converter" {
    import { RoomEvent } from "models/events/RoomEvent";
    /**
     * Wraps a room event into a more suitable container.
     * @param {any} event The event object to wrap.
     * @returns {RoomEvent<any>} An instance of the most suitable container for the event.
     * @category Matrix events
     */
    export function wrapRoomEvent(event: any): RoomEvent<any>;
}
declare module "models/events/AliasesEvent" {
    import { StateEvent } from "models/events/RoomEvent";
    /**
     * The content definition for m.room.aliases events
     * @category Matrix event contents
     * @see AliasesEvent
     */
    export interface AliasesEventContent {
        /**
         * The aliases this domain has published to the room.
         */
        aliases: string[];
    }
    /**
     * Represents an m.room.aliases state event
     * @category Matrix events
     */
    export class AliasesEvent extends StateEvent<AliasesEventContent> {
        constructor(event: any);
        /**
         * The domain the aliases belong to.
         */
        get forDomain(): string;
        /**
         * The aliases the domain has published to the room.
         */
        get aliases(): string[];
    }
}
declare module "models/events/CanonicalAliasEvent" {
    import { StateEvent } from "models/events/RoomEvent";
    /**
     * The content definition for m.room.canonical_alias events
     * @category Matrix event contents
     * @see CanonicalAliasEvent
     */
    export interface CanonicalAliasEventContent {
        /**
         * The canonical alias for the room.
         */
        alias: string;
    }
    /**
     * Represents an m.room.canonical_alias state event
     * @category Matrix events
     */
    export class CanonicalAliasEvent extends StateEvent<CanonicalAliasEventContent> {
        constructor(event: any);
        /**
         * The alias the room is considering canonical
         */
        get aliases(): string;
    }
}
declare module "models/events/CreateEvent" {
    import { StateEvent } from "models/events/RoomEvent";
    /**
     * Information about the previous room.
     * @category Matrix event info
     * @see CreateEventContent
     */
    export interface PreviousRoomInfo {
        /**
         * The old room ID.
         */
        room_id: string;
        /**
         * The last known event ID in the old room.
         */
        event_id: string;
    }
    /**
     * The content definition for m.room.create events
     * @category Matrix event contents
     * @see CreateEvent
     */
    export interface CreateEventContent {
        /**
         * The user ID who created the room.
         */
        creator: string;
        /**
         * Whether or not this room is federated. Default true.
         */
        "m.federate"?: boolean;
        /**
         * The version of the room. Default "1".
         */
        room_version?: string;
        /**
         * Information about the old room.
         */
        predecessor?: PreviousRoomInfo;
    }
    /**
     * Represents an m.room.create state event
     * @category Matrix events
     */
    export class CreateEvent extends StateEvent<CreateEventContent> {
        constructor(event: any);
        /**
         * The user ID who created the room.
         */
        get creator(): string;
        /**
         * The version of the room. Defaults to "1".
         */
        get version(): string;
        /**
         * Whether or not the room is federated. Default true (federated).
         */
        get federated(): boolean;
    }
}
declare module "models/events/JoinRulesEvent" {
    import { StateEvent } from "models/events/RoomEvent";
    /**
     * The types of join rules that are valid in Matrix.
     * @category Matrix event info
     * @see JoinRulesEventContent
     */
    export type JoinRule = "public" | "knock" | "invite" | "private";
    /**
     * The content definition for m.room.join_rules events
     * @category Matrix event contents
     * @see JoinRulesEvent
     */
    export interface JoinRulesEventContent {
        /**
         * The join rule for the room.
         */
        join_rule: JoinRule;
    }
    /**
     * Represents an m.room.join_rules state event
     * @category Matrix events
     */
    export class JoinRulesEvent extends StateEvent<JoinRulesEventContent> {
        constructor(event: any);
        /**
         * The join rule for the room.
         */
        get rule(): JoinRule;
    }
}
declare module "models/events/PowerLevelsEvent" {
    import { StateEvent } from "models/events/RoomEvent";
    /**
     * The content definition for m.room.power_levels events
     * @category Matrix event contents
     * @see PowerLevelsEvent
     */
    export interface PowerLevelsEventContent {
        /**
         * The power level required to ban. Default 50.
         */
        ban?: number;
        /**
         * A map of event types to the power level required to send them.
         */
        events?: {
            [eventType: string]: number;
        };
        /**
         * The power level required to send events in the room. Default 50.
         */
        events_default?: number;
        /**
         * The power level required to invite users to the room. Default 50.
         */
        invite?: number;
        /**
         * The power level required to kick users from the room. Default 50.
         */
        kick?: number;
        /**
         * The power level required to redact other people's events in the room. Default 50.
         */
        redact?: number;
        /**
         * The power level required to send state events in the room. Default 50.
         */
        state_default?: number;
        /**
         * A map of user IDs to power levels.
         */
        users?: {
            [userId: string]: number;
        };
        /**
         * The power level of users not listed in `users`. Default 0.
         */
        users_default?: number;
        /**
         * Power levels required to send certain kinds of notifications.
         */
        notifications?: {
            /**
             * The power level required to send "@room" notifications. Default 50.
             */
            room?: number;
        };
    }
    /**
     * Represents an m.room.power_levels state event
     * @category Matrix events
     */
    export class PowerLevelsEvent extends StateEvent<PowerLevelsEventContent> {
        constructor(event: any);
        /**
         * The power level required to ban users.
         */
        get banLevel(): number;
        /**
         * The power level required to invite users.
         */
        get inviteLevel(): number;
        /**
         * The power level required to kick users.
         */
        get kickLevel(): number;
        /**
         * The power level required to redact messages sent by other users.
         */
        get redactLevel(): number;
        /**
         * The power level required to send "@room" notifications.
         */
        get notifyWholeRoomLevel(): number;
        /**
         * The default power level for users.
         */
        get defaultUserLevel(): number;
        /**
         * The default power level required to send state events.
         */
        get defaultStateEventLevel(): number;
        /**
         * The default power level required to send room events.
         */
        get defaultEventLevel(): number;
    }
}
declare module "models/events/RedactionEvent" {
    import { RoomEvent } from "models/events/RoomEvent";
    /**
     * The content definition for m.room.redaction events
     * @category Matrix event contents
     * @see RedactionEvent
     */
    export interface RedactionEventContent {
        /**
         * The event ID or IDs this event redacts.
         */
        redacts?: string | string[];
    }
    /**
     * Represents an m.room.redaction room event
     * @category Matrix events
     */
    export class RedactionEvent extends RoomEvent<RedactionEventContent> {
        constructor(event: any);
        /**
         * The event ID this event redacts.
         * @deprecated It is possible for multiple events to be redacted depending on the room version.
         */
        get redactsEventId(): string;
        /**
         * The event IDs this event redacts.
         */
        get redactsEventIds(): string[];
    }
}
declare module "models/events/PinnedEventsEvent" {
    import { StateEvent } from "models/events/RoomEvent";
    /**
     * The content definition for m.room.pinned_events events
     * @category Matrix event contents
     * @see PinnedEventsEvent
     */
    export interface PinnedEventsEventContent {
        /**
         * The event IDs that are pinned in the room.
         */
        pinned: string[];
    }
    /**
     * Represents an m.room.pinned_events state event
     * @category Matrix events
     */
    export class PinnedEventsEvent extends StateEvent<PinnedEventsEventContent> {
        constructor(event: any);
        /**
         * The event IDs that are pinned in the room.
         */
        get pinnedEventIds(): string[];
    }
}
declare module "models/events/RoomAvatarEvent" {
    import { StateEvent } from "models/events/RoomEvent";
    import { DimensionalFileInfo } from "models/events/MessageEvent";
    /**
     * The content definition for m.room.avatar events
     * @category Matrix event contents
     * @see RoomAvatarEvent
     */
    export interface RoomAvatarEventContent {
        /**
         * The URL to the image for the avatar of the room.
         */
        url: string;
        /**
         * Optional information about the avatar.
         */
        info?: DimensionalFileInfo;
    }
    /**
     * Represents an m.room.avatar state event
     * @category Matrix events
     */
    export class RoomAvatarEvent extends StateEvent<RoomAvatarEventContent> {
        constructor(event: any);
        /**
         * The URL for the avatar of the room.
         */
        get avatarUrl(): string;
    }
}
declare module "models/events/RoomNameEvent" {
    import { StateEvent } from "models/events/RoomEvent";
    /**
     * The content definition for m.room.name events
     * @category Matrix event contents
     * @see RoomNameEvent
     */
    export interface RoomNameEventContent {
        name: string;
    }
    /**
     * Represents an m.room.name state event
     * @category Matrix events
     */
    export class RoomNameEvent extends StateEvent<RoomNameEventContent> {
        constructor(event: any);
        /**
         * The name of the room.
         */
        get name(): string;
    }
}
declare module "models/events/RoomTopicEvent" {
    import { StateEvent } from "models/events/RoomEvent";
    /**
     * The content definition for m.room.topic events
     * @category Matrix event contents
     * @see RoomTopicEvent
     */
    export interface RoomTopicEventContent {
        topic: string;
    }
    /**
     * Represents an m.room.topic state event
     * @category Matrix events
     */
    export class RoomTopicEvent extends StateEvent<RoomTopicEventContent> {
        constructor(event: any);
        /**
         * The topic of the room.
         */
        get topic(): string;
    }
}
declare module "preprocessors/IPreprocessor" {
    import { MatrixClient } from "MatrixClient";
    import { EventKind } from "index";
    /**
     * Represents a preprocessor.
     * @category Preprocessors
     */
    export interface IPreprocessor {
        /**
         * Gets the types of events this preprocessor supports.
         */
        getSupportedEventTypes(): string[];
        /**
         * Processes an event, modifying it in-place if needed.
         * @param {any} event The event that should be processed.
         * @param {MatrixClient} client The Matrix client that is providing the event.
         * @param {EventKind|null|undefined} kind Optional kind identifier for an event. When not
         * supplied, the event is assumed to be a RoomEvent.
         * @returns {Promise<any>} Resolved when the event is has been modified. The resolved
         * value is ignored.
         */
        processEvent(event: any, client: MatrixClient, kind?: EventKind): Promise<any>;
    }
}
declare module "preprocessors/RichRepliesPreprocessor" {
    import { MatrixClient } from "MatrixClient";
    import { IPreprocessor } from "preprocessors/IPreprocessor";
    import { EventKind } from "index";
    /**
     * Metadata for a rich reply. Usually stored under the "mx_richreply"
     * field of an event (at the top level).
     * @category Preprocessors
     * @see RichRepliesPreprocessor
     */
    export interface IRichReplyMetadata {
        /**
         * If true, the preprocessor found some inconsistencies in the reply
         * information that does not match the specification. For example,
         * this may indicate that a reply was sent without an HTML component.
         */
        wasLenient: boolean;
        /**
         * The event ID the event references. May be an empty string if
         * wasLenient is true.
         */
        parentEventId: string;
        /**
         * The fallback plain text the preprocessor found. May be an empty
         * string if wasLenient is true. The prefix characters to indicate
         * this is a fallback will have already been removed.
         */
        fallbackPlainBody: string;
        /**
         * The fallback HTML the processor found. May be an empty string if
         * wasLenient is true. The fallback structure will have already been
         * removed, leaving just the original assumed HTML.
         */
        fallbackHtmlBody: string;
        /**
         * The user ID that sent the parent event, as determined by the fallback
         * text. This should not be relied upon for anything serious, and instead
         * the preprocessor should be configured to fetch the real event to
         * populate the realEvent property. May be an empty string if wasLenient
         * is true.
         */
        fallbackSender: string;
        /**
         * If the preprocessor is configured to fetch event content, this field
         * will contain the event as reported by the homeserver. May be null if
         * wasLenient is true.
         */
        realEvent: any;
    }
    /**
     * Processes rich replies found in events. This automatically strips
     * the fallback representation from events, providing the information
     * as a top level "mx_richreply" key. The "mx_richreply" property may
     * be casted to the type IRichReplyMetadata.
     * @category Preprocessors
     */
    export class RichRepliesPreprocessor implements IPreprocessor {
        private fetchRealEventContents;
        /**
         * Creates a new rich replies preprocessor.
         * @param fetchRealEventContents If enabled, this preprocessor will
         * attempt to get the real event contents and append them to the event
         * information.
         */
        constructor(fetchRealEventContents?: boolean);
        getSupportedEventTypes(): string[];
        processEvent(event: any, client: MatrixClient, kind?: EventKind): Promise<any>;
    }
}
declare module "storage/SimpleFsStorageProvider" {
    import { IStorageProvider } from "storage/IStorageProvider";
    import { IFilterInfo } from "IFilter";
    import { IAppserviceStorageProvider } from "storage/IAppserviceStorageProvider";
    /**
     * A storage provider that uses the disk to store information.
     * @category Storage providers
     */
    export class SimpleFsStorageProvider implements IStorageProvider, IAppserviceStorageProvider {
        private trackTransactionsInMemory;
        private maxInMemoryTransactions;
        private db;
        private completedTransactions;
        /**
         * Creates a new simple file system storage provider.
         * @param {string} filename The file name (typically 'storage.json') to store data within.
         * @param {boolean} trackTransactionsInMemory True (default) to track all received appservice transactions rather than on disk.
         * @param {int} maxInMemoryTransactions The maximum number of transactions to hold in memory before rotating the oldest out. Defaults to 20.
         */
        constructor(filename: string, trackTransactionsInMemory?: boolean, maxInMemoryTransactions?: number);
        setSyncToken(token: string | null): void;
        getSyncToken(): string | null;
        setFilter(filter: IFilterInfo): void;
        getFilter(): IFilterInfo;
        addRegisteredUser(userId: string): void;
        isUserRegistered(userId: string): boolean;
        isTransactionCompleted(transactionId: string): boolean;
        setTransactionCompleted(transactionId: string): void;
        readValue(key: string): string | null | undefined;
        storeValue(key: string, value: string): void;
        storageForUser(userId: string): IStorageProvider;
    }
}
declare module "strategies/AppserviceJoinRoomStrategy" {
    import { IJoinRoomStrategy } from "strategies/JoinRoomStrategy";
    import { Appservice } from "appservice/Appservice";
    /**
     * A join strategy for application services that proxies joins to an underlying join
     * strategy while also trying to use the appservice's bot user to invite the underlying
     * user if needed.
     * @category Join strategies
     */
    export class AppserviceJoinRoomStrategy implements IJoinRoomStrategy {
        private underlyingStrategy;
        private appservice;
        constructor(underlyingStrategy: IJoinRoomStrategy, appservice: Appservice);
        joinRoom(roomIdOrAlias: string, userId: string, apiCall: (roomIdOrAlias: string) => Promise<string>): Promise<string>;
    }
}
declare module "MatrixAuth" {
    import { MatrixClient } from "MatrixClient";
    /**
     * Functions for interacting with Matrix prior to having an access token. Intended
     * to be used for logging in/registering to get a MatrixClient instance.
     *
     * By design, this limits the options used to create the MatrixClient. To specify
     * custom elements to the client, get the access token from the returned client
     * and create a new MatrixClient instance. Due to the nature of Matrix, it is
     * also recommended to use the homeserverUrl from the generated MatrixClient as
     * it may be different from that given to the MatrixAuth class.
     */
    export class MatrixAuth {
        private homeserverUrl;
        /**
         * Creates a new MatrixAuth class for creating a MatrixClient
         * @param {string} homeserverUrl The homeserver URL to authenticate against.
         */
        constructor(homeserverUrl: string);
        /**
         * Generate a client with no access token so we can reuse the doRequest
         * logic already written.
         */
        private createTemplateClient;
        /**
         * Performs simple registration using a password for the account. This will
         * assume the server supports the m.login.password flow for registration, and
         * will attempt to complete only that stage. The caller is expected to determine
         * if the homeserver supports registration prior to invocation.
         * @param {string} localpart The localpart (username) to register
         * @param {string} password The password to register with
         * @param {string} deviceName The name of the newly created device. Optional.
         * @returns {Promise<MatrixClient>} Resolves to a logged-in MatrixClient
         */
        passwordRegister(localpart: string, password: string, deviceName?: string): Promise<MatrixClient>;
        /**
         * Performs simple password login with the homeserver. The caller is
         * expected to confirm if the homeserver supports this login flow prior
         * to invocation.
         * @param {string} username The username (localpart or user ID) to log in with
         * @param {string} password The password for the account
         * @param {string} deviceName The name of the newly created device. Optional.
         * @returns {Promise<MatrixClient>} Resolves to a logged-in MatrixClient
         */
        passwordLogin(username: string, password: string, deviceName?: string): Promise<MatrixClient>;
    }
}
declare module "models/unstable/MediaInfo" {
    /**
     * This interface implements the schema defined in [MSC2380](https://github.com/matrix-org/matrix-doc/pull/2380).
     */
    export interface MSC2380MediaInfo {
        content_type: string;
        width: number | undefined;
        height: number | undefined;
        size: number;
        thumbnails?: {
            width: number;
            height: number;
            ready: boolean;
        }[];
        duration?: number;
    }
}
declare module "UnstableApis" {
    import { MatrixClient } from "MatrixClient";
    import { MSC2380MediaInfo } from "models/unstable/MediaInfo";
    /**
     * Represents a profile for a group
     * @category Unstable APIs
     */
    export interface GroupProfile {
        /**
         * The name of the group
         */
        name: string;
        /**
         * The avatar for the group. Must be a MSC URI.
         */
        avatar_url: string;
        /**
         * The short description for the group. Equivalent to a room's topic.
         */
        short_description: string;
        /**
         * The long description for the group. Most clients will support HTML
         * in this.
         */
        long_description: string;
    }
    /**
     * Unstable APIs that shouldn't be used in most circumstances.
     * @category Unstable APIs
     */
    export class UnstableApis {
        private client;
        constructor(client: MatrixClient);
        /**
         * Gets the local room aliases that are published for a given room.
         * @param {string} roomId The room ID to get local aliases for.
         * @returns {Promise<string[]>} Resolves to the aliases on the room, or an empty array.
         * @deprecated Relies on MSC2432 endpoint.
         */
        getRoomAliases(roomId: string): Promise<string[]>;
        /**
         * Creates a group.
         * @param {string} localpart The localpart for the group
         * @return {Promise<string>} Resolves to the created group ID.
         */
        createGroup(localpart: string): Promise<string>;
        /**
         * Invites a user to the group.
         * @param {string} groupId The group ID to invite the user to.
         * @param {string} userId The user ID to invite to the group.
         * @return {Promise<"join" | "invite" | "reject">} Resolves to the invite state for
         *  the user. This is normally "invite", but may be "join" or "reject" if the user's
         *  homeserver accepted/rejected the invite right away.
         */
        inviteUserToGroup(groupId: string, userId: string): Promise<"join" | "invite" | "reject">;
        /**
         * Kicks a user from a group.
         * @param {string} groupId The group ID to kick the user from.
         * @param {string} userId The user ID to kick from the group.
         * @return {Promise<any>} Resolves when completed.
         */
        kickUserFromGroup(groupId: string, userId: string): Promise<any>;
        /**
         * Updates a group's profile
         * @param {string} groupId The group ID to update.
         * @param {GroupProfile} profile The profile to update the group with.
         * @return {Promise<any>} Resolves when completed.
         */
        setGroupProfile(groupId: string, profile: GroupProfile): Promise<any>;
        /**
         * Sets a group's join policy to either be publicly joinable (open) or
         * require an invite (invite).
         * @param {string} groupId The group ID to set the policy for.
         * @param {"open" | "invite"} policy The policy to set.
         * @return {Promise<any>} Resolves when completed.
         */
        setGroupJoinPolicy(groupId: string, policy: "open" | "invite"): Promise<any>;
        /**
         * Adds a room to a group.
         * @param {string} groupId The group ID to add the room to.
         * @param {string} roomId The room ID to add to the group.
         * @param {boolean} isPublic Whether this group-room association is visible to non-members. Optional. Defaults to true.
         * @return {Promise<any>} Resolves when completed.
         */
        addRoomToGroup(groupId: string, roomId: string, isPublic?: boolean): Promise<any>;
        /**
         * Updates the visibility of a room in a group.
         * @param {string} groupId The group ID of the room to update.
         * @param {string} roomId The room ID of the room to update.
         * @param {boolean} isPublic Whether this group-room association is visible to non-members.
         * @return {Promise<any>} Resolves when completed.
         */
        updateGroupRoomVisibility(groupId: string, roomId: string, isPublic: boolean): Promise<any>;
        /**
         * Removes a room from a group.
         * @param {string} groupId The group ID to remove the room from.
         * @param {string} roomId The room ID to remove from the group.
         * @return {Promise<any>} Resolves when completed.
         */
        removeRoomFromGroup(groupId: string, roomId: string): Promise<any>;
        /**
         * Gets a group's profile.
         * @param {string} groupId The group ID to fetch the profile of.
         * @return {Promise<GroupProfile>} Resolves to the profile of the group.
         */
        getGroupProfile(groupId: string): Promise<GroupProfile>;
        /**
         * Gets the users in a group.
         * @param {string} groupId The group ID of which to get the users.
         * @return {Promise<any[]>} Resolves to an array of all the users in the group.
         */
        getGroupUsers(groupId: string): Promise<any[]>;
        /**
         * Gets the invited users of a group.
         * @param {string} groupId The group ID of which to get the invited users.
         * @return {Promise<any[]>} Resolves to an array of all the users invited to the group.
         */
        getGroupInvitedUsers(groupId: string): Promise<any[]>;
        /**
         * Gets the rooms of a group.
         * @param {string} groupId The group ID of which to get all the rooms.
         * @return {Promise<any[]>} Resolves to an array of all the rooms of the group.
         */
        getGroupRooms(groupId: string): Promise<any[]>;
        /**
         * Accepts an invite to a group.
         * @param {string} groupId The group ID of which to accept the invite of.
         * @return {Promise<any>} Resolves when completed.
         */
        acceptGroupInvite(groupId: string): Promise<any>;
        /**
         * Joins a group.
         * @param {string} groupId The group ID to join.
         * @return {Promise<any>} Resolves when completed.
         */
        joinGroup(groupId: string): Promise<any>;
        /**
         * Leaves a group.
         * @param {string} groupId The group ID of the group to leave.
         * @return {Promise<any>} Resolves when completed.
         */
        leaveGroup(groupId: string): Promise<any>;
        /**
         * Sets the publicity of a group.
         * @param {string} groupId The group ID to set the publicity of.
         * @param {boolean} publicise If the group should be publicised.
         * @return {Promise<any>} Resolves when completed.
         */
        setGroupPublicity(groupId: string, publicise: boolean): Promise<any>;
        /**
         * Gets all group IDs joined.
         * @return {Promise<string[]>} Resolves to the group IDs of the joined groups.
         */
        getJoinedGroups(): Promise<string[]>;
        /**
         * Gets the group IDs that the specified user has publicised.
         * @param {string} userId The user ID to fetch the publicised groups of.
         * @return {Promise<string[]>} Resolves to the publicised group IDs of that user.
         */
        getPublicisedGroups(userId: string): Promise<string[]>;
        /**
         * Adds a reaction to an event. The contract for this function may change in the future.
         * @param {string} roomId The room ID to react in
         * @param {string} eventId The event ID to react against, in the given room
         * @param {string} emoji The emoji to react with
         * @returns {Promise<string>} Resolves to the event ID of the reaction
         */
        addReactionToEvent(roomId: string, eventId: string, emoji: string): Promise<string>;
        /**
         * Get relations for a given event.
         * @param {string} roomId The room ID to for the given event.
         * @param {string} eventId The event ID to list reacations for.
         * @param {string?} relationType The type of reaction (e.g. `m.room.member`) to filter for. Optional.
         * @param {string?} eventType The type of event to look for (e.g. `m.room.member`). Optional.
         * @returns {Promise<{original_event: any, chunk: any[]}>} Resolves a object containing the original event, and a chunk of relations
         */
        getRelationsForEvent(roomId: string, eventId: string, relationType?: string, eventType?: string): Promise<{
            original_event: any;
            chunk: any[];
        }>;
        /**
         * Get information about a media item. Implements MSC2380
         * @param {string} mxc The MXC to get information about.
         * @returns {Promise<MSC2380MediaInfo>} Resolves a object containing the media information.
         */
        getMediaInfo(mxcUrl: string): Promise<MSC2380MediaInfo>;
    }
}
declare module "PantalaimonClient" {
    /**
     * A client specifically designed to interact with Pantalaimon instead of
     * a Matrix homeserver. The key part of this is managing the access token
     * and username/password for interacting with Pantalaimon.
     *
     * If the storage provider given claims to have an access token for
     * this client, it will be used even if Pantalaimon considers it invalid.
     *
     * Expected usage:
     * <code>
     *     const storage = new SimpleFsStorageProvider("storage/bot.json");
     *     const pantalaimon = new PantalaimonClient("http://localhost:8008", storage);
     *
     *     // Note that the credentials will only be used if there is no available access token.
     *     const client = await pantalaimon.createClientWithCredentials("username", "password");
     * </code>
     */
    import { IStorageProvider } from "storage/IStorageProvider";
    import { MatrixClient } from "MatrixClient";
    /**
     * Supporting functions for interacting with a Pantalaimon instance.
     * @category Encryption
     */
    export class PantalaimonClient {
        private homeserverUrl;
        private storageProvider;
        /**
         * Creates a new PantalaimonClient class for interacting with Pantalaimon. The storage
         * provider given will also be used in the client.
         * @param {string} homeserverUrl The homeserver (Pantalaimon) URL to interact with.
         * @param {IStorageProvider} storageProvider The storage provider to back the client with.
         */
        constructor(homeserverUrl: string, storageProvider: IStorageProvider);
        /**
         * Authenticates and creates the Pantalaimon-capable client. The username and password given
         * are only used if the storage provider does not reveal an access token.
         * @param {string} username The username to log in with, if needed.
         * @param {string} password The password to log in with, if needed.
         * @returns {Promise<MatrixClient>} Resolves to a MatrixClient ready for interacting with Pantalaimon.
         */
        createClientWithCredentials(username: string, password: string): Promise<MatrixClient>;
    }
}
declare module "SynchronousMatrixClient" {
    import { MatrixClient } from "MatrixClient";
    /**
     * A MatrixClient class that handles events in sync for the /sync loop, instead
     * of trying to push its way through the /sync loop as fast as possible. It is
     * intended that the consumer extend this class and override the onWhatever()
     * functions it needs. All of the onWhatever() functions have a default behaviour
     * of doing nothing.
     */
    export abstract class SynchronousMatrixClient extends MatrixClient {
        /**
         * Creates a new SynchronousMatrixClient. Note that this accepts a MatrixClient, though
         * much of the class's properties are not brought over. Always convert your MatrixClient
         * instance to a SynchronousMatrixClient as soon as possible to avoid diversion in which
         * properties are proxied over.
         * @param {MatrixClient} baseClient The client to wrap.
         */
        protected constructor(baseClient: MatrixClient);
        private handleEvent;
        protected startSyncInternal(): Promise<any>;
        /**
         * Handles the `account_data` event raised by the client.
         * @param {any} event The account data event.
         * @returns {Promise<any>} Resolves when complete.
         */
        protected onAccountData(event: any): Promise<any>;
        /**
         * Handles the `room.account_data` event raised by the client.
         * @param {string} roomId The Room ID the account data applies to.
         * @param {any} event The room account data event.
         * @returns {Promise<any>} Resolves when complete.
         */
        protected onRoomAccountData(roomId: string, event: any): Promise<any>;
        /**
         * Handles the `room.leave` event raised by the client.
         * @param {string} roomId The Room ID the event happened in.
         * @param {any} event The event.
         * @returns {Promise<any>} Resolves when complete.
         */
        protected onRoomLeave(roomId: string, event: any): Promise<any>;
        /**
         * Handles the `room.invite` event raised by the client.
         * @param {string} roomId The Room ID the event happened in.
         * @param {any} event The event.
         * @returns {Promise<any>} Resolves when complete.
         */
        protected onRoomInvite(roomId: string, event: any): Promise<any>;
        /**
         * Handles the `room.join` event raised by the client.
         * @param {string} roomId The Room ID the event happened in.
         * @param {any} event The event.
         * @returns {Promise<any>} Resolves when complete.
         */
        protected onRoomJoin(roomId: string, event: any): Promise<any>;
        /**
         * Handles the `room.message` event raised by the client.
         * @param {string} roomId The Room ID the event happened in.
         * @param {any} event The event.
         * @returns {Promise<any>} Resolves when complete.
         */
        protected onRoomMessage(roomId: string, event: any): Promise<any>;
        /**
         * Handles the `room.archived` event raised by the client.
         * @param {string} roomId The Room ID the event happened in.
         * @param {any} event The event.
         * @returns {Promise<any>} Resolves when complete.
         */
        protected onRoomArchived(roomId: string, event: any): Promise<any>;
        /**
         * Handles the `room.upgraded` event raised by the client.
         * @param {string} roomId The Room ID the event happened in.
         * @param {any} event The event.
         * @returns {Promise<any>} Resolves when complete.
         */
        protected onRoomUpgraded(roomId: string, event: any): Promise<any>;
        /**
         * Handles the `room.event` event raised by the client.
         * @param {string} roomId The Room ID the event happened in.
         * @param {any} event The event.
         * @returns {Promise<any>} Resolves when complete.
         */
        protected onRoomEvent(roomId: string, event: any): Promise<any>;
    }
}
declare module "SynapseAdminApis" {
    import { MatrixClient } from "MatrixClient";
    /**
     * Information about a user on Synapse.
     * @category Admin APIs
     */
    export interface SynapseUser {
        /***
         * The display name of the user, if set.
         */
        displayname?: string;
        /**
         * A set of 3PIDs for the user.
         */
        threepids?: {
            medium: string;
            address: string;
        }[];
        /**
         * The avatar URL (usually MXC URI) for the user, if set.
         */
        avatar_url?: string;
        /**
         * Whether or not the user is a Synapse administrator.
         */
        admin?: boolean;
        /**
         * Whether or not the user is deactivated.
         */
        deactivated?: boolean;
    }
    /**
     * Added information to include when updating/creating a user.
     * @category Admin APIs
     */
    export interface SynapseUserProperties extends SynapseUser {
        /**
         * The password for the user. Leave undefined to leave unchanged.
         */
        password?: string;
    }
    /**
     * Information about a user on Synapse.
     * @category Admin APIs
     */
    export interface SynapseUserListing {
        /**
         * User ID.
         */
        name: string;
        /**
         * Whether or not the user is a guest. 1 is true, 0 is false.
         */
        is_guest: number;
        /**
         * Whether or not the user is an admin. 1 is true, 0 is false.
         */
        admin: number;
        /**
         * Whether or not the user is deactivated. 1 is true, 0 is false.
         */
        deactivated: number;
        /**
         * The type of user, if relevant.
         */
        user_type: string | null;
        /**
         * The hash of the user's password, if relevant.
         */
        password_hash: string | null;
        /**
         * The display name of the user, if set.
         */
        displayname: string | null;
        /**
         * The avatar for the user, if set.
         */
        avatar_url: string | null;
    }
    /**
     * A resulting list of users on Synapse.
     * @category Admin APIs
     */
    export interface SynapseUserList {
        /**
         * A set of users matching the criteria.
         */
        users: SynapseUserListing[];
        /**
         * The token to use to get the next set of users.
         */
        next_token: string;
        /**
         * The total number of users on the Synapse instance.
         */
        total: number;
    }
    /**
     * Information about a room on Synapse.
     * @category Admin APIs
     */
    export interface SynapseRoomListing {
        room_id: string;
        name: string;
        canonical_alias: string;
        joined_members: number;
        joined_local_members: number;
        version: string;
        creator: string;
        encryption: string;
        federatable: boolean;
        public: boolean;
        join_rules: string;
        guest_access: string;
        history_visibility: string;
        state_events: number;
    }
    /**
     * A resulting list of rooms on Synapse.
     * @category Admin APIs
     */
    export interface SynapseRoomList {
        rooms: SynapseRoomListing[];
        offset: string;
        total_rooms: number;
        next_batch: string;
        prev_batch: string;
    }
    /**
     * Available properties on a Synapse room listing to order by.
     * @category Admin APIs
     */
    export enum SynapseRoomProperty {
        Name = "name",
        CanonicalAlias = "canonical_alias",
        JoinedMembers = "joined_members",
        JoinedLocalMembers = "joined_local_members",
        Version = "version",
        Creator = "creator",
        Encryption = "encryption",
        CanFederate = "federatable",
        IsPublic = "public",
        JoinRules = "join_rules",
        GuestAccess = "guest_access",
        HistoryVisibility = "history_visibility",
        NumStateEvents = "state_events"
    }
    /**
     * Access to various administrative APIs specifically available in Synapse.
     * @category Admin APIs
     */
    export class SynapseAdminApis {
        private client;
        constructor(client: MatrixClient);
        /**
         * Get information about a user. The client making the request must be an admin user.
         * @param {string} userId The user ID to check.
         * @returns {Promise<SynapseUser>} The resulting Synapse user record
         */
        getUser(userId: string): Promise<SynapseUser>;
        /**
         * Create or update a given user on a Synapse server. The
         * client making the request must be an admin user.
         * @param {string} userId The user ID to check.
         * @param {SynapseUserProperties} opts Options to set when creating or updating the user.
         * @returns {Promise<SynapseUser>} The resulting Synapse user record
         */
        upsertUser(userId: string, opts?: SynapseUserProperties): Promise<SynapseUser>;
        /**
         * Get a list of users registered with Synapse, optionally filtered by some criteria. The
         * client making the request must be an admin user.
         * @param {string} from The token to continue listing users from.
         * @param {number} limit The maximum number of users to request.
         * @param {string} name Optional localpart or display name filter for results.
         * @param {boolean} guests Whether or not to include guest accounts. Default true.
         * @param {boolean} deactivated Whether or not to include deactivated accounts. Default false.
         * @returns {Promise<SynapseUserList>} A batch of user results.
         */
        listUsers(from?: string, limit?: number, name?: string, guests?: boolean, deactivated?: boolean): Promise<SynapseUserList>;
        /**
         * Determines if the given user is a Synapse server administrator for this homeserver. The
         * client making the request must be an admin user themselves (check with `isSelfAdmin`)
         * @param {string} userId The user ID to check.
         * @returns {Promise<boolean>} Resolves to true if the user is an admin, false otherwise.
         * Throws if there's an error.
         */
        isAdmin(userId: string): Promise<boolean>;
        /**
         * Determines if the current user is an admin for the Synapse homeserver.
         * @returns {Promise<boolean>} Resolve to true if the user is an admin, false otherwise.
         * Throws if there's an error.
         */
        isSelfAdmin(): Promise<boolean>;
        /**
         * Lists the rooms on the server.
         * @param {string} searchTerm A term to search for in the room names
         * @param {string} from A previous batch token to search from
         * @param {number} limit The maximum number of rooms to return
         * @param {SynapseRoomProperty} orderBy A property of rooms to order by
         * @param {boolean} reverseOrder True to reverse the orderBy direction.
         * @returns {Promise<SynapseRoomList>} Resolves to the server's rooms, ordered and filtered.
         */
        listRooms(searchTerm?: string, from?: string, limit?: number, orderBy?: SynapseRoomProperty, reverseOrder?: boolean): Promise<SynapseRoomList>;
        /**
         * Gets a list of state events in a room.
         * @param {string} roomId The room ID to get state for.
         * @returns {Promise<any[]>} Resolves to the room's state events.
         */
        getRoomState(roomId: string): Promise<any[]>;
        /**
         * Deletes a room from the server, purging all record of it.
         * @param {string} roomId The room to delete.
         * @returns {Promise} Resolves when complete.
         */
        deleteRoom(roomId: string): Promise<void>;
    }
}
declare module "index" {
    export * from "appservice/Appservice";
    export * from "appservice/Intent";
    export * from "appservice/MatrixBridge";
    export * from "appservice/http_responses";
    export * from "appservice/UnstableAppserviceApis";
    export * from "e2ee/RoomTracker";
    export * from "e2ee/CryptoClient";
    export * from "e2ee/decorators";
    export * from "helpers/RichReply";
    export * from "helpers/MentionPill";
    export * from "helpers/Permalinks";
    export * from "helpers/MatrixGlob";
    export * from "helpers/ProfileCache";
    export * from "helpers/MatrixEntity";
    export * from "helpers/UnpaddedBase64";
    export * from "logging/ConsoleLogger";
    export * from "logging/RichConsoleLogger";
    export * from "logging/ILogger";
    export * from "logging/LogService";
    export * from "metrics/contexts";
    export * from "metrics/names";
    export * from "metrics/decorators";
    export * from "metrics/IMetricListener";
    export * from "metrics/Metrics";
    export * from "mixins/AutojoinRoomsMixin";
    export * from "mixins/AutojoinUpgradedRoomsMixin";
    export * from "models/Presence";
    export * from "models/MatrixProfile";
    export * from "models/EventContext";
    export * from "models/PowerLevelBounds";
    export * from "models/OpenIDConnect";
    export * from "models/Policies";
    export * from "models/Threepid";
    export * from "models/Spaces";
    export * from "models/IdentityServerModels";
    export * from "models/Crypto";
    export * from "models/MSC2176";
    export * from "models/Account";
    export * from "models/events/EventKind";
    export * from "models/events/converter";
    export * from "models/events/InvalidEventError";
    export * from "models/events/Event";
    export * from "models/events/RoomEvent";
    export * from "models/events/PresenceEvent";
    export * from "models/events/MembershipEvent";
    export * from "models/events/MessageEvent";
    export * from "models/events/AliasesEvent";
    export * from "models/events/CanonicalAliasEvent";
    export * from "models/events/CreateEvent";
    export * from "models/events/JoinRulesEvent";
    export * from "models/events/PowerLevelsEvent";
    export * from "models/events/RedactionEvent";
    export * from "models/events/PinnedEventsEvent";
    export * from "models/events/RoomAvatarEvent";
    export * from "models/events/RoomNameEvent";
    export * from "models/events/RoomTopicEvent";
    export * from "models/events/SpaceChildEvent";
    export * from "models/events/EncryptionEvent";
    export * from "models/events/EncryptedRoomEvent";
    export * from "preprocessors/IPreprocessor";
    export * from "preprocessors/RichRepliesPreprocessor";
    export * from "storage/IAppserviceStorageProvider";
    export * from "storage/IStorageProvider";
    export * from "storage/MemoryStorageProvider";
    export * from "storage/SimpleFsStorageProvider";
    export * from "storage/ICryptoStorageProvider";
    export * from "strategies/AppserviceJoinRoomStrategy";
    export * from "strategies/JoinRoomStrategy";
    export * from "identity/IdentityClient";
    export * from "IFilter";
    export * from "MatrixClient";
    export * from "MatrixAuth";
    export * from "UnstableApis";
    export * from "AdminApis";
    export * from "request";
    export * from "PantalaimonClient";
    export * from "SynchronousMatrixClient";
    export * from "SynapseAdminApis";
    export * from "simple-validation";
    export * from "b64";
}
declare module "strategies/JoinRoomStrategy" {
    export interface IJoinRoomStrategy {
        joinRoom(roomIdOrAlias: string, userId: string, apiCall: (roomIdOrAlias: string) => Promise<string>): Promise<string>;
    }
    /**
     * A join strategy that keeps trying to join the room on a set interval.
     * @category Join strategies
     */
    export class SimpleRetryJoinStrategy implements IJoinRoomStrategy {
        private schedule;
        joinRoom(roomIdOrAlias: string, userId: string, apiCall: (roomIdOrAlias: string) => Promise<string>): Promise<string>;
    }
}
declare module "models/PowerLevelAction" {
    /**
     * Actions that can be guarded by power levels.
     */
    export enum PowerLevelAction {
        /**
         * Power level required to ban other users.
         */
        Ban = "ban",
        /**
         * Power level required to kick other users.
         */
        Kick = "kick",
        /**
         * Power level required to redact events sent by other users. Users can redact
         * their own messages regardless of this power level requirement, unless forbidden
         * by the `events` section of the power levels content.
         */
        RedactEvents = "redact",
        /**
         * Power level required to invite other users.
         */
        Invite = "invite",
        /**
         * Power level required to notify the whole room with "@room".
         */
        NotifyRoom = "notifications.room"
    }
}
declare module "MatrixClient" {
    import { EventEmitter } from "events";
    import { IStorageProvider } from "storage/IStorageProvider";
    import { IJoinRoomStrategy } from "strategies/JoinRoomStrategy";
    import { UnstableApis } from "UnstableApis";
    import { IPreprocessor } from "preprocessors/IPreprocessor";
    import { Metrics } from "metrics/Metrics";
    import { AdminApis } from "AdminApis";
    import { Presence } from "models/Presence";
    import { Membership, MembershipEvent } from "models/events/MembershipEvent";
    import { EventContext } from "models/EventContext";
    import { PowerLevelBounds } from "models/PowerLevelBounds";
    import { IdentityClient } from "identity/IdentityClient";
    import { OpenIDConnectToken } from "models/OpenIDConnect";
    import { Space, SpaceCreateOptions } from "models/Spaces";
    import { PowerLevelAction } from "models/PowerLevelAction";
    import { CryptoClient } from "e2ee/CryptoClient";
    import { DeviceKeyAlgorithm, DeviceKeyLabel, EncryptionAlgorithm, FallbackKey, MultiUserDeviceListResponse, OTKAlgorithm, OTKClaimResponse, OTKCounts, OTKs, OwnUserDevice } from "models/Crypto";
    import { ICryptoStorageProvider } from "storage/ICryptoStorageProvider";
    import { IWhoAmI } from "models/Account";
    /**
     * A client that is capable of interacting with a matrix homeserver.
     */
    export class MatrixClient extends EventEmitter {
        readonly homeserverUrl: string;
        readonly accessToken: string;
        private storage;
        readonly cryptoStore: ICryptoStorageProvider;
        /**
         * The presence status to use while syncing. The valid values are "online" to set the account as online,
         * "offline" to set the user as offline, "unavailable" for marking the user away, and null for not setting
         * an explicit presence (the default).
         *
         * Has no effect if the client is not syncing. Does not apply until the next sync request.
         */
        syncingPresence: "online" | "offline" | "unavailable" | null;
        /**
         * The number of milliseconds to wait for new events for on the next sync.
         *
         * Has no effect if the client is not syncing. Does not apply until the next sync request.
         */
        syncingTimeout: number;
        /**
         * The crypto manager instance for this client. Generally speaking, this shouldn't
         * need to be accessed but is made available.
         *
         * Will be null/undefined if crypto is not possible.
         */
        readonly crypto: CryptoClient;
        private userId;
        private requestId;
        private lastJoinedRoomIds;
        private impersonatedUserId;
        private impersonatedDeviceId;
        private joinStrategy;
        private eventProcessors;
        private filterId;
        private stopSyncing;
        private metricsInstance;
        private unstableApisInstance;
        /**
         * Set this to true to have the client only persist the sync token after the sync
         * has been processed successfully. Note that if this is true then when the sync
         * loop throws an error the client will not persist a token.
         */
        protected persistTokenAfterSync: boolean;
        /**
         * Creates a new matrix client
         * @param {string} homeserverUrl The homeserver's client-server API URL
         * @param {string} accessToken The access token for the homeserver
         * @param {IStorageProvider} storage The storage provider to use. Defaults to MemoryStorageProvider.
         * @param {ICryptoStorageProvider} cryptoStore Optional crypto storage provider to use. If not supplied,
         * end-to-end encryption will not be functional in this client.
         */
        constructor(homeserverUrl: string, accessToken: string, storage?: IStorageProvider, cryptoStore?: ICryptoStorageProvider);
        /**
         * The storage provider for this client. Direct access is usually not required.
         */
        get storageProvider(): IStorageProvider;
        /**
         * The metrics instance for this client
         */
        get metrics(): Metrics;
        /**
         * Assigns a new metrics instance, overwriting the old one.
         * @param {Metrics} metrics The new metrics instance.
         */
        set metrics(metrics: Metrics);
        /**
         * Gets the unstable API access class. This is generally not recommended to be
         * used by clients.
         * @return {UnstableApis} The unstable API access class.
         */
        get unstableApis(): UnstableApis;
        /**
         * Gets the admin API access class.
         * @return {AdminApis} The admin API access class.
         */
        get adminApis(): AdminApis;
        /**
         * Sets a user ID to impersonate as. This will assume that the access token for this client
         * is for an application service, and that the userId given is within the reach of the
         * application service. Setting this to null will stop future impersonation. The user ID is
         * assumed to already be valid
         * @param {string} userId The user ID to masquerade as, or `null` to clear masquerading.
         * @param {string} deviceId Optional device ID to impersonate under the given user, if supported
         * by the server. Check the whoami response after setting.
         */
        impersonateUserId(userId: string | null, deviceId?: string): void;
        /**
         * Acquires an identity server client for communicating with an identity server. Note that
         * this will automatically do the login portion to establish a usable token with the identity
         * server provided, but it will not automatically accept any terms of service.
         *
         * The identity server name provided will in future be resolved to a server address - for now
         * that resolution is assumed to be prefixing the name with `https://`.
         * @param {string} identityServerName The domain of the identity server to connect to.
         * @returns {Promise<IdentityClient>} Resolves to a prepared identity client.
         */
        getIdentityServerClient(identityServerName: string): Promise<IdentityClient>;
        /**
         * Sets the strategy to use for when joinRoom is called on this client
         * @param {IJoinRoomStrategy} strategy The strategy to use, or null to use none
         */
        setJoinStrategy(strategy: IJoinRoomStrategy): void;
        /**
         * Adds a preprocessor to the event pipeline. When this client encounters an event, it
         * will try to run it through the preprocessors it can in the order they were added.
         * @param {IPreprocessor} preprocessor the preprocessor to add
         */
        addPreprocessor(preprocessor: IPreprocessor): void;
        private processEvent;
        /**
         * Retrieves an OpenID Connect token from the homeserver for the current user.
         * @returns {Promise<OpenIDConnectToken>} Resolves to the token.
         */
        getOpenIDConnectToken(): Promise<OpenIDConnectToken>;
        /**
         * Retrieves content from account data.
         * @param {string} eventType The type of account data to retrieve.
         * @returns {Promise<any>} Resolves to the content of that account data.
         */
        getAccountData<T>(eventType: string): Promise<T>;
        /**
         * Retrieves content from room account data.
         * @param {string} eventType The type of room account data to retrieve.
         * @param {string} roomId The room to read the account data from.
         * @returns {Promise<any>} Resolves to the content of that account data.
         */
        getRoomAccountData<T>(eventType: string, roomId: string): Promise<T>;
        /**
         * Retrieves content from account data. If the account data request throws an error,
         * this simply returns the default provided.
         * @param {string} eventType The type of account data to retrieve.
         * @param {any} defaultContent The default value. Defaults to null.
         * @returns {Promise<any>} Resolves to the content of that account data, or the default.
         */
        getSafeAccountData<T>(eventType: string, defaultContent?: T): Promise<T>;
        /**
         * Retrieves content from room account data. If the account data request throws an error,
         * this simply returns the default provided.
         * @param {string} eventType The type of room account data to retrieve.
         * @param {string} roomId The room to read the account data from.
         * @param {any} defaultContent The default value. Defaults to null.
         * @returns {Promise<any>} Resolves to the content of that room account data, or the default.
         */
        getSafeRoomAccountData<T>(eventType: string, roomId: string, defaultContent?: T): Promise<T>;
        /**
         * Sets account data.
         * @param {string} eventType The type of account data to set
         * @param {any} content The content to set
         * @returns {Promise<any>} Resolves when updated
         */
        setAccountData(eventType: string, content: any): Promise<any>;
        /**
         * Sets room account data.
         * @param {string} eventType The type of room account data to set
         * @param {string} roomId The room to set account data in
         * @param {any} content The content to set
         * @returns {Promise<any>} Resolves when updated
         */
        setRoomAccountData(eventType: string, roomId: string, content: any): Promise<any>;
        /**
         * Gets the presence information for the current user.
         * @returns {Promise<Presence>} Resolves to the presence status of the user.
         */
        getPresenceStatus(): Promise<Presence>;
        /**
         * Gets the presence information for a given user.
         * @param {string} userId The user ID to look up the presence of.
         * @returns {Promise<Presence>} Resolves to the presence status of the user.
         */
        getPresenceStatusFor(userId: string): Promise<Presence>;
        /**
         * Sets the presence status for the current user.
         * @param {"online"|"offline"|"unavailable"} presence The new presence state for the user.
         * @param {string} statusMessage Optional status message to include with the presence.
         * @returns {Promise<any>} Resolves when complete.
         */
        setPresenceStatus(presence: "online" | "offline" | "unavailable", statusMessage?: string): Promise<any>;
        /**
         * Gets a published alias for the given room. These are supplied by the room admins
         * and should point to the room, but may not. This is primarily intended to be used
         * in the context of rendering a mention (pill) for a room.
         * @param {string} roomIdOrAlias The room ID or alias to get an alias for.
         * @returns {Promise<string>} Resolves to a published room alias, or falsey if none found.
         */
        getPublishedAlias(roomIdOrAlias: string): Promise<string>;
        /**
         * Adds a new room alias to the room directory
         * @param {string} alias The alias to add (eg: "#my-room:matrix.org")
         * @param {string} roomId The room ID to add the alias to
         * @returns {Promise} resolves when the alias has been added
         */
        createRoomAlias(alias: string, roomId: string): Promise<any>;
        /**
         * Removes a room alias from the room directory
         * @param {string} alias The alias to remove
         * @returns {Promise} resolves when the alias has been deleted
         */
        deleteRoomAlias(alias: string): Promise<any>;
        /**
         * Sets the visibility of a room in the directory.
         * @param {string} roomId The room ID to manipulate the visibility of
         * @param {"public" | "private"} visibility The visibility to set for the room
         * @return {Promise} resolves when the visibility has been updated
         */
        setDirectoryVisibility(roomId: string, visibility: "public" | "private"): Promise<any>;
        /**
         * Gets the visibility of a room in the directory.
         * @param {string} roomId The room ID to query the visibility of
         * @return {Promise<"public"|"private">} The visibility of the room
         */
        getDirectoryVisibility(roomId: string): Promise<"public" | "private">;
        /**
         * Resolves a room ID or alias to a room ID. If the given ID or alias looks like a room ID
         * already, it will be returned as-is. If the room ID or alias looks like a room alias, it
         * will be resolved to a room ID if possible. If the room ID or alias is neither, an error
         * will be raised.
         * @param {string} roomIdOrAlias the room ID or alias to resolve to a room ID
         * @returns {Promise<string>} resolves to the room ID
         */
        resolveRoom(roomIdOrAlias: string): Promise<string>;
        /**
         * Does a room directory lookup for a given room alias
         * @param {string} roomAlias the room alias to look up in the room directory
         * @returns {Promise<RoomDirectoryLookupResponse>} resolves to the room's information
         */
        lookupRoomAlias(roomAlias: string): Promise<RoomDirectoryLookupResponse>;
        /**
         * Invites a user to a room.
         * @param {string} userId the user ID to invite
         * @param {string} roomId the room ID to invite the user to
         * @returns {Promise<any>} resolves when completed
         */
        inviteUser(userId: any, roomId: any): Promise<any>;
        /**
         * Kicks a user from a room.
         * @param {string} userId the user ID to kick
         * @param {string} roomId the room ID to kick the user in
         * @param {string?} reason optional reason for the kick
         * @returns {Promise<any>} resolves when completed
         */
        kickUser(userId: any, roomId: any, reason?: any): Promise<any>;
        /**
         * Bans a user from a room.
         * @param {string} userId the user ID to ban
         * @param {string} roomId the room ID to set the ban in
         * @param {string?} reason optional reason for the ban
         * @returns {Promise<any>} resolves when completed
         */
        banUser(userId: any, roomId: any, reason?: any): Promise<any>;
        /**
         * Unbans a user in a room.
         * @param {string} userId the user ID to unban
         * @param {string} roomId the room ID to lift the ban in
         * @returns {Promise<any>} resolves when completed
         */
        unbanUser(userId: any, roomId: any): Promise<any>;
        /**
         * Gets the current user ID for this client
         * @returns {Promise<string>} The user ID of this client
         */
        getUserId(): Promise<string>;
        /**
         * Gets the user's information from the server directly.
         * @returns {Promise<IWhoAmI>} The "who am I" response.
         */
        getWhoAmI(): Promise<IWhoAmI>;
        /**
         * Stops the client from syncing.
         */
        stop(): void;
        /**
         * Starts syncing the client with an optional filter
         * @param {any} filter The filter to use, or null for none
         * @returns {Promise<any>} Resolves when the client has started syncing
         */
        start(filter?: any): Promise<any>;
        protected startSyncInternal(): Promise<any>;
        protected startSync(emitFn?: (emitEventType: string, ...payload: any[]) => Promise<any>): Promise<void>;
        protected doSync(token: string): Promise<any>;
        protected processSync(raw: any, emitFn?: (emitEventType: string, ...payload: any[]) => Promise<any>): Promise<any>;
        /**
         * Gets an event for a room. If the event is encrypted, and the client supports encryption,
         * and the room is encrypted, then this will return a decrypted event.
         * @param {string} roomId the room ID to get the event in
         * @param {string} eventId the event ID to look up
         * @returns {Promise<any>} resolves to the found event
         */
        getEvent(roomId: string, eventId: string): Promise<any>;
        /**
         * Gets an event for a room. Returned as a raw event.
         * @param {string} roomId the room ID to get the event in
         * @param {string} eventId the event ID to look up
         * @returns {Promise<any>} resolves to the found event
         */
        getRawEvent(roomId: string, eventId: string): Promise<any>;
        /**
         * Gets the room state for the given room. Returned as raw events.
         * @param {string} roomId the room ID to get state for
         * @returns {Promise<any[]>} resolves to the room's state
         */
        getRoomState(roomId: string): Promise<any[]>;
        /**
         * Gets the state events for a given room of a given type under the given state key.
         * @param {string} roomId the room ID
         * @param {string} type the event type
         * @param {String} stateKey the state key, falsey if not needed
         * @returns {Promise<any|any[]>} resolves to the state event(s)
         * @deprecated It is not possible to get an array of events - use getRoomStateEvent instead
         */
        getRoomStateEvents(roomId: any, type: any, stateKey: any): Promise<any | any[]>;
        /**
         * Gets a state event for a given room of a given type under the given state key.
         * @param {string} roomId the room ID
         * @param {string} type the event type
         * @param {String} stateKey the state key
         * @returns {Promise<any>} resolves to the state event
         */
        getRoomStateEvent(roomId: any, type: any, stateKey: any): Promise<any>;
        /**
         * Gets the context surrounding an event.
         * @param {string} roomId The room ID to get the context in.
         * @param {string} eventId The event ID to get the context of.
         * @param {number} limit The maximum number of events to return on either side of the event.
         * @returns {Promise<EventContext>} The context of the event
         */
        getEventContext(roomId: string, eventId: string, limit?: number): Promise<EventContext>;
        /**
         * Gets the profile for a given user
         * @param {string} userId the user ID to lookup
         * @returns {Promise<any>} the profile of the user
         */
        getUserProfile(userId: string): Promise<any>;
        /**
         * Sets a new display name for the user.
         * @param {string} displayName the new display name for the user, or null to clear
         * @returns {Promise<any>} resolves when complete
         */
        setDisplayName(displayName: string): Promise<any>;
        /**
         * Sets a new avatar url for the user.
         * @param {string} avatarUrl the new avatar URL for the user, in the form of a Matrix Content URI
         * @returns {Promise<any>} resolves when complete
         */
        setAvatarUrl(avatarUrl: string): Promise<any>;
        /**
         * Joins the given room
         * @param {string} roomIdOrAlias the room ID or alias to join
         * @param {string[]} viaServers the server names to try and join through
         * @returns {Promise<string>} resolves to the joined room ID
         */
        joinRoom(roomIdOrAlias: string, viaServers?: string[]): Promise<string>;
        /**
         * Gets a list of joined room IDs
         * @returns {Promise<string[]>} resolves to a list of room IDs the client participates in
         */
        getJoinedRooms(): Promise<string[]>;
        /**
         * Gets the joined members in a room. The client must be in the room to make this request.
         * @param {string} roomId The room ID to get the joined members of.
         * @returns {Promise<string>} The joined user IDs in the room
         */
        getJoinedRoomMembers(roomId: string): Promise<string[]>;
        /**
         * Gets the joined members in a room, as an object mapping userIds to profiles. The client must be in the room to make this request.
         * @param {string} roomId The room ID to get the joined members of.
         * @returns {Object} The joined user IDs in the room as an object mapped to a set of profiles.
         */
        getJoinedRoomMembersWithProfiles(roomId: string): Promise<{
            [userId: string]: {
                display_name?: string;
                avatar_url?: string;
            };
        }>;
        /**
         * Gets the membership events of users in the room. Defaults to all membership
         * types, though this can be controlled with the membership and notMembership
         * arguments. To change the point in time, use the batchToken.
         * @param {string} roomId The room ID to get members in.
         * @param {string} batchToken The point in time to get members at (or null for 'now')
         * @param {string[]} membership The membership kinds to search for.
         * @param {string[]} notMembership The membership kinds to not search for.
         * @returns {Promise<any[]>} Resolves to the membership events of the users in the room.
         */
        getRoomMembers(roomId: string, batchToken?: string, membership?: Membership[], notMembership?: Membership[]): Promise<MembershipEvent[]>;
        /**
         * Leaves the given room
         * @param {string} roomId the room ID to leave
         * @returns {Promise<any>} resolves when left
         */
        leaveRoom(roomId: string): Promise<any>;
        /**
         * Sends a read receipt for an event in a room
         * @param {string} roomId the room ID to send the receipt to
         * @param {string} eventId the event ID to set the receipt at
         * @returns {Promise<any>} resolves when the receipt has been sent
         */
        sendReadReceipt(roomId: string, eventId: string): Promise<any>;
        /**
         * Sets the typing status of the current user in a room
         * @param {string} roomId the room ID the user is typing in
         * @param {boolean} typing is the user currently typing
         * @param {number} timeout how long should the server preserve the typing state, in milliseconds
         * @returns {Promise<any>} resolves when the typing state has been set
         */
        setTyping(roomId: string, typing: boolean, timeout?: number): Promise<any>;
        /**
         * Replies to a given event with the given text. The event is sent with a msgtype of m.text.
         * The message will be encrypted if the client supports encryption and the room is encrypted.
         * @param {string} roomId the room ID to reply in
         * @param {any} event the event to reply to
         * @param {string} text the text to reply with
         * @param {string} html the HTML to reply with, or falsey to use the `text`
         * @returns {Promise<string>} resolves to the event ID which was sent
         */
        replyText(roomId: string, event: any, text: string, html?: string): Promise<string>;
        /**
         * Replies to a given event with the given HTML. The event is sent with a msgtype of m.text.
         * The message will be encrypted if the client supports encryption and the room is encrypted.
         * @param {string} roomId the room ID to reply in
         * @param {any} event the event to reply to
         * @param {string} html the HTML to reply with.
         * @returns {Promise<string>} resolves to the event ID which was sent
         */
        replyHtmlText(roomId: string, event: any, html: string): Promise<string>;
        /**
         * Replies to a given event with the given text. The event is sent with a msgtype of m.notice.
         * The message will be encrypted if the client supports encryption and the room is encrypted.
         * @param {string} roomId the room ID to reply in
         * @param {any} event the event to reply to
         * @param {string} text the text to reply with
         * @param {string} html the HTML to reply with, or falsey to use the `text`
         * @returns {Promise<string>} resolves to the event ID which was sent
         */
        replyNotice(roomId: string, event: any, text: string, html?: string): Promise<string>;
        /**
         * Replies to a given event with the given HTML. The event is sent with a msgtype of m.notice.
         * The message will be encrypted if the client supports encryption and the room is encrypted.
         * @param {string} roomId the room ID to reply in
         * @param {any} event the event to reply to
         * @param {string} html the HTML to reply with.
         * @returns {Promise<string>} resolves to the event ID which was sent
         */
        replyHtmlNotice(roomId: string, event: any, html: string): Promise<string>;
        /**
         * Sends a notice to the given room. The message will be encrypted if the client supports
         * encryption and the room is encrypted.
         * @param {string} roomId the room ID to send the notice to
         * @param {string} text the text to send
         * @returns {Promise<string>} resolves to the event ID that represents the message
         */
        sendNotice(roomId: string, text: string): Promise<string>;
        /**
         * Sends a notice to the given room with HTML content. The message will be encrypted if the client supports
         * encryption and the room is encrypted.
         * @param {string} roomId the room ID to send the notice to
         * @param {string} html the HTML to send
         * @returns {Promise<string>} resolves to the event ID that represents the message
         */
        sendHtmlNotice(roomId: string, html: string): Promise<string>;
        /**
         * Sends a text message to the given room. The message will be encrypted if the client supports
         * encryption and the room is encrypted.
         * @param {string} roomId the room ID to send the text to
         * @param {string} text the text to send
         * @returns {Promise<string>} resolves to the event ID that represents the message
         */
        sendText(roomId: string, text: string): Promise<string>;
        /**
         * Sends a text message to the given room with HTML content. The message will be encrypted if the client supports
         * encryption and the room is encrypted.
         * @param {string} roomId the room ID to send the text to
         * @param {string} html the HTML to send
         * @returns {Promise<string>} resolves to the event ID that represents the message
         */
        sendHtmlText(roomId: string, html: string): Promise<string>;
        /**
         * Sends a message to the given room. The message will be encrypted if the client supports
         * encryption and the room is encrypted.
         * @param {string} roomId the room ID to send the message to
         * @param {object} content the event content to send
         * @returns {Promise<string>} resolves to the event ID that represents the message
         */
        sendMessage(roomId: string, content: any): Promise<string>;
        /**
         * Sends an event to the given room. This will encrypt the event before sending if the room is
         * encrypted and the client supports encryption. Use sendRawEvent() to avoid this behaviour.
         * @param {string} roomId the room ID to send the event to
         * @param {string} eventType the type of event to send
         * @param {string} content the event body to send
         * @returns {Promise<string>} resolves to the event ID that represents the event
         */
        sendEvent(roomId: string, eventType: string, content: any): Promise<string>;
        /**
         * Sends an event to the given room.
         * @param {string} roomId the room ID to send the event to
         * @param {string} eventType the type of event to send
         * @param {string} content the event body to send
         * @returns {Promise<string>} resolves to the event ID that represents the event
         */
        sendRawEvent(roomId: string, eventType: string, content: any): Promise<string>;
        /**
         * Sends a state event to the given room
         * @param {string} roomId the room ID to send the event to
         * @param {string} type the event type to send
         * @param {string} stateKey the state key to send, should not be null
         * @param {string} content the event body to send
         * @returns {Promise<string>} resolves to the event ID that represents the message
         */
        sendStateEvent(roomId: string, type: string, stateKey: string, content: any): Promise<string>;
        /**
         * Redact an event in a given room
         * @param {string} roomId the room ID to send the redaction to
         * @param {string} eventId the event ID to redact
         * @param {String} reason an optional reason for redacting the event
         * @returns {Promise<string>} resolves to the event ID that represents the redaction
         */
        redactEvent(roomId: string, eventId: string, reason?: string | null): Promise<string>;
        /**
         * Creates a room. This does not break out the various options for creating a room
         * due to the large number of possibilities. See the /createRoom endpoint in the
         * spec for more information on what to provide for `properties`. Note that creating
         * a room may cause the bot/appservice to raise a join event.
         * @param {any} properties the properties of the room. See the spec for more information
         * @returns {Promise<string>} resolves to the room ID that represents the room
         */
        createRoom(properties?: any): Promise<string>;
        /**
         * Checks if a given user has a required power level required to send the given event.
         * @param {string} userId the user ID to check the power level of
         * @param {string} roomId the room ID to check the power level in
         * @param {string} eventType the event type to look for in the `events` property of the power levels
         * @param {boolean} isState true to indicate the event is intended to be a state event
         * @returns {Promise<boolean>} resolves to true if the user has the required power level, resolves to false otherwise
         */
        userHasPowerLevelFor(userId: string, roomId: string, eventType: string, isState: boolean): Promise<boolean>;
        /**
         * Checks if a given user has a required power level to perform the given action
         * @param {string} userId the user ID to check the power level of
         * @param {string} roomId the room ID to check the power level in
         * @param {PowerLevelAction} action the action to check power level for
         * @returns {Promise<boolean>} resolves to true if the user has the required power level, resolves to false otherwise
         */
        userHasPowerLevelForAction(userId: string, roomId: string, action: PowerLevelAction): Promise<boolean>;
        /**
         * Determines the boundary conditions for this client's ability to change another user's power level
         * in a given room. This will identify the maximum possible level this client can change the user to,
         * and if that change could even be possible. If the returned object indicates that the client can
         * change the power level of the user, the client is able to set the power level to any value equal
         * to or less than the maximum value.
         * @param {string} targetUserId The user ID to compare against.
         * @param {string} roomId The room ID to compare within.
         * @returns {Promise<PowerLevelBounds>} The bounds of the client's ability to change the user's power level.
         */
        calculatePowerLevelChangeBoundsOn(targetUserId: string, roomId: string): Promise<PowerLevelBounds>;
        /**
         * Sets the power level for a given user ID in the given room. Note that this is not safe to
         * call multiple times concurrently as changes are not atomic. This will throw an error if
         * the user lacks enough permission to change the power level, or if a power level event is
         * missing from the room.
         * @param {string} userId The user ID to change
         * @param {string} roomId The room ID to change the power level in
         * @param {number} newLevel The integer power level to set the user to.
         * @returns {Promise<any>} Resolves when complete.
         */
        setUserPowerLevel(userId: string, roomId: string, newLevel: number): Promise<any>;
        /**
         * Converts a MXC URI to an HTTP URL.
         * @param {string} mxc The MXC URI to convert
         * @returns {string} The HTTP URL for the content.
         */
        mxcToHttp(mxc: string): string;
        /**
         * Converts a MXC URI to an HTTP URL for downsizing the content.
         * @param {string} mxc The MXC URI to convert and downsize.
         * @param {number} width The width, as an integer, for the thumbnail.
         * @param {number} height The height, as an intenger, for the thumbnail.
         * @param {"crop"|"scale"} method Whether to crop or scale (preserve aspect ratio) the content.
         * @returns {string} The HTTP URL for the downsized content.
         */
        mxcToHttpThumbnail(mxc: string, width: number, height: number, method: "crop" | "scale"): string;
        /**
         * Uploads data to the homeserver's media repository. Note that this will <b>not</b> automatically encrypt
         * media as it cannot determine if the media should be encrypted.
         * @param {Buffer} data the content to upload.
         * @param {string} contentType the content type of the file. Defaults to application/octet-stream
         * @param {string} filename the name of the file. Optional.
         * @returns {Promise<string>} resolves to the MXC URI of the content
         */
        uploadContent(data: Buffer, contentType?: string, filename?: string): Promise<string>;
        /**
         * Download content from the homeserver's media repository. Note that this will <b>not</b> automatically decrypt
         * media as it cannot determine if the media is encrypted.
         * @param {string} mxcUrl The MXC URI for the content.
         * @param {string} allowRemote Indicates to the server that it should not attempt to fetch the
         * media if it is deemed remote. This is to prevent routing loops where the server contacts itself.
         * Defaults to true if not provided.
         * @returns {Promise<{data: Buffer, contentType: string}>} Resolves to the downloaded content.
         */
        downloadContent(mxcUrl: string, allowRemote?: boolean): Promise<{
            data: Buffer;
            contentType: string;
        }>;
        /**
         * Uploads data to the homeserver's media repository after downloading it from the
         * provided URL.
         * @param {string} url The URL to download content from.
         * @returns {Promise<string>} Resolves to the MXC URI of the content
         */
        uploadContentFromUrl(url: string): Promise<string>;
        /**
         * Determines the upgrade history for a given room as a doubly-linked list styled structure. Given
         * a room ID in the history of upgrades, the resulting `previous` array will hold any rooms which
         * are older than the given room. The resulting `newer` array will hold any rooms which are newer
         * versions of the room. Both arrays will be defined, but may be empty individually. Element zero
         * of each will always be the nearest to the given room ID and the last element will be the furthest
         * from the room. The given room will never be in either array.
         * @param {string} roomId the room ID to get the history of
         * @returns {Promise<{previous: RoomReference[], newer: RoomReference[]}>} Resolves to the room's
         * upgrade history
         */
        getRoomUpgradeHistory(roomId: string): Promise<{
            previous: RoomReference[];
            newer: RoomReference[];
            current: RoomReference;
        }>;
        /**
         * Creates a Space room.
         * @param {SpaceCreateOptions} opts The creation options.
         * @returns {Promise<Space>} Resolves to the created space.
         */
        createSpace(opts: SpaceCreateOptions): Promise<Space>;
        /**
         * Gets a Space.
         * This API does not work with unstable spaces (e.g. org.matrix.msc.1772.space)
         *
         * @throws If the room is not a space or there was an error
         * @returns {Promise<Space>} Resolves to the space.
         */
        getSpace(roomIdOrAlias: string): Promise<Space>;
        /**
         * Uploads new identity keys for the current device.
         * @param {EncryptionAlgorithm[]} algorithms The supported algorithms.
         * @param {Record<DeviceKeyLabel<DeviceKeyAlgorithm, string>, string>} keys The keys for the device.
         * @returns {Promise<OTKCounts>} Resolves to the current One Time Key counts when complete.
         */
        uploadDeviceKeys(algorithms: EncryptionAlgorithm[], keys: Record<DeviceKeyLabel<DeviceKeyAlgorithm, string>, string>): Promise<OTKCounts>;
        /**
         * Uploads One Time Keys for the current device.
         * @param {OTKs} keys The keys to upload.
         * @returns {Promise<OTKCounts>} Resolves to the current One Time Key counts when complete.
         */
        uploadDeviceOneTimeKeys(keys: OTKs): Promise<OTKCounts>;
        /**
         * Gets the current One Time Key counts.
         * @returns {Promise<OTKCounts>} Resolves to the One Time Key counts.
         */
        checkOneTimeKeyCounts(): Promise<OTKCounts>;
        /**
         * Uploads a fallback One Time Key to the server for usage. This will replace the existing fallback
         * key.
         * @param {FallbackKey} fallbackKey The fallback key.
         * @returns {Promise<OTKCounts>} Resolves to the One Time Key counts.
         */
        uploadFallbackKey(fallbackKey: FallbackKey): Promise<OTKCounts>;
        /**
         * Gets <b>unverified</b> device lists for the given users. The caller is expected to validate
         * and verify the device lists, including that the returned devices belong to the claimed users.
         *
         * Failures with federation are reported in the returned object. Users which did not fail a federation
         * lookup but have no devices will not appear in either the failures or in the returned devices.
         *
         * See https://matrix.org/docs/spec/client_server/r0.6.1#post-matrix-client-r0-keys-query for more
         * information.
         * @param {string[]} userIds The user IDs to query.
         * @param {number} federationTimeoutMs The default timeout for requesting devices over federation. Defaults to
         * 10 seconds.
         * @returns {Promise<MultiUserDeviceListResponse>} Resolves to the device list/errors for the requested user IDs.
         */
        getUserDevices(userIds: string[], federationTimeoutMs?: number): Promise<MultiUserDeviceListResponse>;
        /**
         * Gets a device list for the client's own account, with metadata. The devices are not verified
         * in this response, but should be active on the account.
         * @returns {Promise<OwnUserDevice[]>} Resolves to the active devices on the account.
         */
        getOwnDevices(): Promise<OwnUserDevice[]>;
        /**
         * Claims One Time Keys for a set of user devices, returning those keys. The caller is expected to verify
         * and validate the returned keys.
         *
         * Failures with federation are reported in the returned object.
         * @param {Record<string, Record<string, OTKAlgorithm>>} userDeviceMap The map of user IDs to device IDs to
         * OTKAlgorithm to request a claim for.
         * @param {number} federationTimeoutMs The default timeout for claiming keys over federation. Defaults to
         * 10 seconds.
         */
        claimOneTimeKeys(userDeviceMap: Record<string, Record<string, OTKAlgorithm>>, federationTimeoutMs?: number): Promise<OTKClaimResponse>;
        /**
         * Sends to-device messages to the respective users/devices.
         * @param {string} type The message type being sent.
         * @param {Record<string, Record<string, any>>} messages The messages to send, mapped as user ID to
         * device ID (or "*" to denote all of the user's devices) to message payload (content).
         * @returns {Promise<void>} Resolves when complete.
         */
        sendToDevices(type: string, messages: Record<string, Record<string, any>>): Promise<void>;
        /**
         * Performs a web request to the homeserver, applying appropriate authorization headers for
         * this client.
         * @param {"GET"|"POST"|"PUT"|"DELETE"} method The HTTP method to use in the request
         * @param {string} endpoint The endpoint to call. For example: "/_matrix/client/r0/account/whoami"
         * @param {any} qs The query string to send. Optional.
         * @param {any} body The request body to send. Optional. Will be converted to JSON unless the type is a Buffer.
         * @param {number} timeout The number of milliseconds to wait before timing out.
         * @param {boolean} raw If true, the raw response will be returned instead of the response body.
         * @param {string} contentType The content type to send. Only used if the `body` is a Buffer.
         * @param {string} noEncoding Set to true to disable encoding, and return a Buffer. Defaults to false
         * @returns {Promise<any>} Resolves to the response (body), rejected if a non-2xx status code was returned.
         */
        doRequest(method: any, endpoint: any, qs?: any, body?: any, timeout?: number, raw?: boolean, contentType?: string, noEncoding?: boolean): Promise<any>;
    }
    export interface RoomDirectoryLookupResponse {
        roomId: string;
        residentServers: string[];
    }
    export interface RoomReference {
        /**
         * The room ID being referenced
         */
        roomId: string;
        /**
         * The version of the room at the time
         */
        version: string;
        /**
         * If going backwards, the tombstone event ID, otherwise the creation
         * event. If the room can't be verified, this will be null. Will be
         * null if this reference is to the current room.
         */
        refEventId: string;
    }
}
declare module "AdminApis" {
    import { MatrixClient } from "MatrixClient";
    import { SynapseAdminApis } from "SynapseAdminApis";
    /**
     * Whois information about a user.
     * See https://matrix.org/docs/spec/client_server/r0.5.0#get-matrix-client-r0-admin-whois-userid for more information.
     * @category Admin APIs
     */
    export interface WhoisInfo {
        user_id: string;
        devices: {
            [device_id: string]: {
                sessions: [
                    {
                        connections: WhoisConnectionInfo[];
                    }
                ];
            };
        };
    }
    interface WhoisConnectionInfo {
        /**
         * Most recently seen IP address of the session.
         */
        ip: string;
        /**
         * Unix timestamp that the session was last active.
         */
        last_seen: number;
        /**
         * User agent string last seen in the session.
         */
        user_agent: string;
    }
    /**
     * Access to various administrative APIs.
     * @category Admin APIs
     */
    export class AdminApis {
        private client;
        constructor(client: MatrixClient);
        /**
         * Gets access to the Synapse administrative APIs object.
         */
        get synapse(): SynapseAdminApis;
        /**
         * Gets information about a particular user.
         * @param {string} userId the user ID to lookup
         * @returns {Promise<WhoisInfo>} resolves to the whois information
         */
        whoisUser(userId: string): Promise<WhoisInfo>;
    }
}
declare module "storage/CryptexCryptoSecureStorageProvider" {
    import { ICryptoSecureStorageProvider } from "storage/ICryptoSecureStorageProvider";
    import { CryptexOpts } from "cryptex";
    /**
     * A storage provider for secure information relating to crypto, stored using Cryptex.
     * Requires `cryptex` package to be installed.
     *
     * The pickle key is stored as the "pickle_key" secret.
     * @category Storage providers
     * @see https://github.com/TomFrost/Cryptex
     */
    export class CryptexCryptoSecureStorageProvider implements ICryptoSecureStorageProvider {
        private cryptex;
        constructor(opts?: CryptexOpts);
        getPickleKey(): Promise<string>;
        setPickleKey(pickleKey: string): Promise<void>;
    }
}
declare module "storage/NamespacingPostgresCryptoStorageProvider" {
    import { ICryptoStorageProvider } from "storage/ICryptoStorageProvider";
    import { EncryptionEventContent } from "models/events/EncryptionEvent";
    import { IInboundGroupSession, IOlmSession, IOutboundGroupSession, StoredUserDevice, UserDevice } from "models/Crypto";
    import { IAppserviceCryptoStorageProvider } from "storage/IAppserviceStorageProvider";
    import { ICryptoSecureStorageProvider } from "storage/ICryptoSecureStorageProvider";
    /**
     * PostgreSQL crypto storage provider for appservices and clients. Requires `pg-promise` package to be installed.
     * @category Storage providers
     */
    export class NamespacingPostgresCryptoStorageProvider implements IAppserviceCryptoStorageProvider, ICryptoStorageProvider {
        private db;
        private namespace;
        private setup;
        private secureStorage;
        private kvUpsert;
        private kvSelect;
        private roomUpsert;
        private roomSelect;
        private userUpsert;
        private userSelect;
        private userDeviceUpsert;
        private userDevicesDelete;
        private userDevicesSelect;
        private userActiveDevicesSelect;
        private userActiveDeviceSelect;
        private obGroupSessionUpsert;
        private obGroupSessionSelect;
        private obGroupCurrentSessionSelect;
        private obGroupSessionMarkAllInactive;
        private obSentGroupSessionUpsert;
        private obSentSelectLastSent;
        private olmSessionUpsert;
        private olmSessionCurrentSelect;
        private olmSessionSelect;
        private ibGroupSessionUpsert;
        private ibGroupSessionSelect;
        private deMetadataUpsert;
        private deMetadataSelect;
        /**
         * Creates a new PostgreSQL storage provider.
         * @param {string} connectionString A postgresql connection string to the database.
         * @param {ICryptoSecureStorageProvider} secureStorage Secure storage for pickle information.
         * @see https://github.com/vitaly-t/pg-promise/wiki/Connection-Syntax
         */
        constructor(connectionString: string, secureStorage: ICryptoSecureStorageProvider);
        /**
         * Creates a new namespaced storage provider.
         * @param {string} namespace The namespace.
         * @param {NamespacingPostgresCryptoStorageProvider} parent The parent provider.
         */
        constructor(namespace: string, parent: NamespacingPostgresCryptoStorageProvider);
        setDeviceId(deviceId: string): Promise<void>;
        getDeviceId(): Promise<string>;
        setPickleKey(pickleKey: string): Promise<void>;
        getPickleKey(): Promise<string>;
        setPickledAccount(pickled: string): Promise<void>;
        getPickledAccount(): Promise<string>;
        storeRoom(roomId: string, config: Partial<EncryptionEventContent>): Promise<void>;
        getRoom(roomId: string): Promise<Partial<EncryptionEventContent>>;
        setActiveUserDevices(userId: string, devices: UserDevice[]): Promise<void>;
        getActiveUserDevices(userId: string): Promise<UserDevice[]>;
        getActiveUserDevice(userId: string, deviceId: string): Promise<UserDevice>;
        getAllUserDevices(userId: string): Promise<StoredUserDevice[]>;
        flagUsersOutdated(userIds: string[]): Promise<void>;
        isUserOutdated(userId: string): Promise<boolean>;
        storeOutboundGroupSession(session: IOutboundGroupSession): Promise<void>;
        getOutboundGroupSession(sessionId: string, roomId: string): Promise<IOutboundGroupSession>;
        getCurrentOutboundGroupSession(roomId: string): Promise<IOutboundGroupSession>;
        storeSentOutboundGroupSession(session: IOutboundGroupSession, index: number, device: UserDevice): Promise<void>;
        getLastSentOutboundGroupSession(userId: string, deviceId: string, roomId: string): Promise<{
            sessionId: string;
            index: number;
        }>;
        storeOlmSession(userId: string, deviceId: string, session: IOlmSession): Promise<void>;
        getCurrentOlmSession(userId: string, deviceId: string): Promise<IOlmSession>;
        getOlmSessions(userId: string, deviceId: string): Promise<IOlmSession[]>;
        storeInboundGroupSession(session: IInboundGroupSession): Promise<void>;
        getInboundGroupSession(senderUserId: string, senderDeviceId: string, roomId: string, sessionId: string): Promise<IInboundGroupSession>;
        setMessageIndexForEvent(roomId: string, eventId: string, sessionId: string, messageIndex: number): Promise<void>;
        getEventForMessageIndex(roomId: string, sessionId: string, messageIndex: number): Promise<string>;
        storageForUser(userId: string): ICryptoStorageProvider;
    }
}
declare module "storage/NamespacingSqliteCryptoStorageProvider" {
    import { ICryptoStorageProvider } from "storage/ICryptoStorageProvider";
    import { EncryptionEventContent } from "models/events/EncryptionEvent";
    import { IInboundGroupSession, IOlmSession, IOutboundGroupSession, StoredUserDevice, UserDevice } from "models/Crypto";
    import { IAppserviceCryptoStorageProvider } from "storage/IAppserviceStorageProvider";
    /**
     * Sqlite crypto storage provider for appservices and clients. Requires `better-sqlite3` package to be installed.
     * @category Storage providers
     */
    export class NamespacingSqliteCryptoStorageProvider implements IAppserviceCryptoStorageProvider, ICryptoStorageProvider {
        private db;
        private namespace;
        private kvUpsert;
        private kvSelect;
        private roomUpsert;
        private roomSelect;
        private userUpsert;
        private userSelect;
        private userDeviceUpsert;
        private userDevicesDelete;
        private userDevicesSelect;
        private userActiveDevicesSelect;
        private userActiveDeviceSelect;
        private obGroupSessionUpsert;
        private obGroupSessionSelect;
        private obGroupCurrentSessionSelect;
        private obGroupSessionMarkAllInactive;
        private obSentGroupSessionUpsert;
        private obSentSelectLastSent;
        private olmSessionUpsert;
        private olmSessionCurrentSelect;
        private olmSessionSelect;
        private ibGroupSessionUpsert;
        private ibGroupSessionSelect;
        private deMetadataUpsert;
        private deMetadataSelect;
        /**
         * Creates a new Sqlite storage provider.
         * @param {string} path The file path to store the database at. Use ":memory:" to
         * store the database entirely in memory, or an empty string to do the equivalent
         * on the disk.
         */
        constructor(path: string);
        /**
         * Creates a new namespaced storage provider.
         * @param {string} namespace The namespace.
         * @param {NamespacingSqliteCryptoStorageProvider} parent The parent provider.
         */
        constructor(namespace: string, parent: NamespacingSqliteCryptoStorageProvider);
        setDeviceId(deviceId: string): Promise<void>;
        getDeviceId(): Promise<string>;
        setPickleKey(pickleKey: string): Promise<void>;
        getPickleKey(): Promise<string>;
        setPickledAccount(pickled: string): Promise<void>;
        getPickledAccount(): Promise<string>;
        storeRoom(roomId: string, config: Partial<EncryptionEventContent>): Promise<void>;
        getRoom(roomId: string): Promise<Partial<EncryptionEventContent>>;
        setActiveUserDevices(userId: string, devices: UserDevice[]): Promise<void>;
        getActiveUserDevices(userId: string): Promise<UserDevice[]>;
        getActiveUserDevice(userId: string, deviceId: string): Promise<UserDevice>;
        getAllUserDevices(userId: string): Promise<StoredUserDevice[]>;
        flagUsersOutdated(userIds: string[]): Promise<void>;
        isUserOutdated(userId: string): Promise<boolean>;
        storeOutboundGroupSession(session: IOutboundGroupSession): Promise<void>;
        getOutboundGroupSession(sessionId: string, roomId: string): Promise<IOutboundGroupSession>;
        getCurrentOutboundGroupSession(roomId: string): Promise<IOutboundGroupSession>;
        storeSentOutboundGroupSession(session: IOutboundGroupSession, index: number, device: UserDevice): Promise<void>;
        getLastSentOutboundGroupSession(userId: string, deviceId: string, roomId: string): Promise<{
            sessionId: string;
            index: number;
        }>;
        storeOlmSession(userId: string, deviceId: string, session: IOlmSession): Promise<void>;
        getCurrentOlmSession(userId: string, deviceId: string): Promise<IOlmSession>;
        getOlmSessions(userId: string, deviceId: string): Promise<IOlmSession[]>;
        storeInboundGroupSession(session: IInboundGroupSession): Promise<void>;
        getInboundGroupSession(senderUserId: string, senderDeviceId: string, roomId: string, sessionId: string): Promise<IInboundGroupSession>;
        setMessageIndexForEvent(roomId: string, eventId: string, sessionId: string, messageIndex: number): Promise<void>;
        getEventForMessageIndex(roomId: string, sessionId: string, messageIndex: number): Promise<string>;
        storageForUser(userId: string): ICryptoStorageProvider;
        /**
         * Closes the crypto store. Primarily for testing purposes.
         */
        close(): Promise<void>;
    }
}
declare module "storage/SqliteCryptoStorageProvider" {
    import { ICryptoStorageProvider } from "storage/ICryptoStorageProvider";
    import { EncryptionEventContent } from "models/events/EncryptionEvent";
    import { IInboundGroupSession, IOlmSession, IOutboundGroupSession, StoredUserDevice, UserDevice } from "models/Crypto";
    /**
     * Sqlite crypto storage provider. Requires `better-sqlite3` package to be installed.
     * @category Storage providers
     * @deprecated Use `NamespacingSqliteCryptoStorageProvider` instead
     */
    export class SqliteCryptoStorageProvider implements ICryptoStorageProvider {
        private db;
        private kvUpsert;
        private kvSelect;
        private roomUpsert;
        private roomSelect;
        private userUpsert;
        private userSelect;
        private userDeviceUpsert;
        private userDevicesDelete;
        private userDevicesSelect;
        private userActiveDevicesSelect;
        private userActiveDeviceSelect;
        private obGroupSessionUpsert;
        private obGroupSessionSelect;
        private obGroupCurrentSessionSelect;
        private obGroupSessionMarkAllInactive;
        private obSentGroupSessionUpsert;
        private obSentSelectLastSent;
        private olmSessionUpsert;
        private olmSessionCurrentSelect;
        private olmSessionSelect;
        private ibGroupSessionUpsert;
        private ibGroupSessionSelect;
        private deMetadataUpsert;
        private deMetadataSelect;
        /**
         * Creates a new Sqlite storage provider.
         * @param {string} path The file path to store the database at. Use ":memory:" to
         * store the database entirely in memory, or an empty string to do the equivalent
         * on the disk.
         */
        constructor(path: string);
        setDeviceId(deviceId: string): Promise<void>;
        getDeviceId(): Promise<string>;
        setPickleKey(pickleKey: string): Promise<void>;
        getPickleKey(): Promise<string>;
        setPickledAccount(pickled: string): Promise<void>;
        getPickledAccount(): Promise<string>;
        storeRoom(roomId: string, config: Partial<EncryptionEventContent>): Promise<void>;
        getRoom(roomId: string): Promise<Partial<EncryptionEventContent>>;
        setActiveUserDevices(userId: string, devices: UserDevice[]): Promise<void>;
        getActiveUserDevices(userId: string): Promise<UserDevice[]>;
        getActiveUserDevice(userId: string, deviceId: string): Promise<UserDevice>;
        getAllUserDevices(userId: string): Promise<StoredUserDevice[]>;
        flagUsersOutdated(userIds: string[]): Promise<void>;
        isUserOutdated(userId: string): Promise<boolean>;
        storeOutboundGroupSession(session: IOutboundGroupSession): Promise<void>;
        getOutboundGroupSession(sessionId: string, roomId: string): Promise<IOutboundGroupSession>;
        getCurrentOutboundGroupSession(roomId: string): Promise<IOutboundGroupSession>;
        storeSentOutboundGroupSession(session: IOutboundGroupSession, index: number, device: UserDevice): Promise<void>;
        getLastSentOutboundGroupSession(userId: string, deviceId: string, roomId: string): Promise<{
            sessionId: string;
            index: number;
        }>;
        storeOlmSession(userId: string, deviceId: string, session: IOlmSession): Promise<void>;
        getCurrentOlmSession(userId: string, deviceId: string): Promise<IOlmSession>;
        getOlmSessions(userId: string, deviceId: string): Promise<IOlmSession[]>;
        storeInboundGroupSession(session: IInboundGroupSession): Promise<void>;
        getInboundGroupSession(senderUserId: string, senderDeviceId: string, roomId: string, sessionId: string): Promise<IInboundGroupSession>;
        setMessageIndexForEvent(roomId: string, eventId: string, sessionId: string, messageIndex: number): Promise<void>;
        getEventForMessageIndex(roomId: string, sessionId: string, messageIndex: number): Promise<string>;
        /**
         * Closes the crypto store. Primarily for testing purposes.
         */
        close(): Promise<void>;
    }
}
