"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SyncV3MatrixClient = void 0;
const MatrixClient_1 = require("../MatrixClient");
const LogService_1 = require("../logging/LogService");
const V3List_1 = require("./V3List");
/**
 * A MatrixClient class which attempts to use Sync V3 instead of the normal sync protocol
 * on the server.
 *
 * This class is considered <b>UNSTABLE</b> and may be removed at any time. This class will
 * not check for server support before attempting to use it.
 *
 * @category Unstable: Sync V3
 */
class SyncV3MatrixClient extends MatrixClient_1.MatrixClient {
    /**
     * Creates a new matrix client
     * @param {string} homeserverUrl The homeserver's client-server API URL
     * @param {string} accessToken The access token for the homeserver
     * @param {IStorageProvider} storage The storage provider to use.
     * @param {SortBehaviour[]} sortBehaviours The list sorting behaviour to use.
     */
    constructor(homeserverUrl, accessToken, storage, sortBehaviours = [V3List_1.SortBehaviour.Name]) {
        super(homeserverUrl, accessToken, storage);
        this.lists = [];
        this.lists = [
            new V3List_1.V3List(V3List_1.ListBehaviour.JoinedOnly, sortBehaviours),
            new V3List_1.V3List(V3List_1.ListBehaviour.InvitedOnly, sortBehaviours),
            new V3List_1.V3List(V3List_1.ListBehaviour.DMsOnly, sortBehaviours),
        ];
    }
    get joinedList() {
        return this.lists[0];
    }
    get invitedList() {
        return this.lists[1];
    }
    get directMessagesList() {
        return this.lists[2];
    }
    startSyncInternal() {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            this.lastPos = yield this.storageProvider.getSyncToken();
            for (let i = 0; i < this.lists.length; i++) {
                const value = JSON.parse((yield this.storageProvider.readValue(`sync_v3_list_${i}`)) || "{}");
                yield this.lists[i].processOperations(this, (_a = value.count) !== null && _a !== void 0 ? _a : 0, (_b = value.ops) !== null && _b !== void 0 ? _b : []);
            }
            this.loopSync();
        });
    }
    loopSync() {
        const promiseWhile = () => __awaiter(this, void 0, void 0, function* () {
            if (this.stopSyncing) {
                LogService_1.LogService.info("MatrixClientLite", "Client stop requested - stopping sync");
                return;
            }
            try {
                const response = yield this.doSyncV3();
                yield this.processSyncV3(response);
                this.lastPos = response.pos;
                yield this.storageProvider.setSyncToken(this.lastPos);
            }
            catch (e) {
                LogService_1.LogService.error("MatrixClientLite", "Error handling sync " + LogService_1.extractRequestError(e));
                const backoffTime = 5000 + Math.random() * (15000 - 5000); // TODO: de-hardcode values. SYNC_BACKOFF_MIN_MS SYNC_BACKOFF_MAX_MS
                LogService_1.LogService.info("MatrixClientLite", `Backing off for ${backoffTime}ms`);
                yield new Promise((r) => setTimeout(r, backoffTime));
            }
            return promiseWhile();
        });
        promiseWhile(); // start loop async
    }
    processSyncV3(sync) {
        return __awaiter(this, void 0, void 0, function* () {
            for (let i = 0; i < this.lists.length; i++) {
                const ops = sync.ops.filter(o => o.list === i);
                yield this.lists[i].processOperations(this, sync.counts[i], ops);
                yield this.storageProvider.storeValue(`sync_v3_list_${i}`, JSON.stringify({
                    ops: this.lists[i].lossySerialized,
                    count: this.lists[i].totalRoomCount,
                }));
            }
        });
    }
    doSyncV3() {
        return __awaiter(this, void 0, void 0, function* () {
            const userId = yield this.getUserId();
            const qs = {};
            if (this.lastPos)
                qs['pos'] = this.lastPos;
            return this.doRequest("POST", "/_matrix/client/unstable/org.matrix.msc3575/sync", qs, {
                lists: this.lists.map(l => l.getDefinitionFor(userId)),
                // TODO: Support extensions for crypto and such
            });
        });
    }
}
exports.SyncV3MatrixClient = SyncV3MatrixClient;
