"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.V3List = exports.CURRENT_USER_STATE_KEY = exports.SortBehaviour = exports.ListBehaviour = void 0;
const V3Room_1 = require("./V3Room");
const operations_1 = require("./operations");
/**
 * @category Unstable: Sync V3
 */
var ListBehaviour;
(function (ListBehaviour) {
    ListBehaviour[ListBehaviour["JoinedOnly"] = 0] = "JoinedOnly";
    ListBehaviour[ListBehaviour["InvitedOnly"] = 1] = "InvitedOnly";
    ListBehaviour[ListBehaviour["DMsOnly"] = 2] = "DMsOnly";
})(ListBehaviour = exports.ListBehaviour || (exports.ListBehaviour = {}));
/**
 * @category Unstable: Sync V3
 */
var SortBehaviour;
(function (SortBehaviour) {
    SortBehaviour["NotificationCount"] = "by_notification_count";
    SortBehaviour["Recency"] = "by_recency";
    SortBehaviour["Name"] = "by_name";
})(SortBehaviour = exports.SortBehaviour || (exports.SortBehaviour = {}));
/**
 * @category Unstable: Sync V3
 */
exports.CURRENT_USER_STATE_KEY = "@CURRENT";
// Unclear if it's even worth changing this to be smaller/bigger, but let's start here.
const WINDOW_SIZE = 1000;
const TIMELINE_LIMIT = 20; // make configurable?
/**
 * @category Unstable: Sync V3
 */
class V3List {
    constructor(behaviour, sort) {
        this.behaviour = behaviour;
        this.sort = sort;
        this.roomsByRange = new Map(); // key is JSON-serialized range
        this.roomMap = new Map(); // key is room ID
        this.totalCount = 0;
    }
    get lossySerialized() {
        return Array.from(this.roomsByRange.entries()).map(([serializedRange, rooms]) => ({
            op: operations_1.Operation.Sync,
            range: JSON.parse(serializedRange),
            rooms: rooms.filter(r => !!r).map(r => r.lossySerialized),
        }));
    }
    get totalRoomCount() {
        return this.totalCount;
    }
    get orderedKnownRooms() {
        // "Just because you can chain functions, doesn't mean you should" ~ CS Instructor
        return Array.from(this.roomsByRange.entries())
            .map(([serializedRange, rooms]) => [JSON.parse(serializedRange)[0], rooms])
            .sort((a, b) => a[0] - b[0])
            .map(e => e[1])
            .reduce((p, c) => [...p, ...c], []);
    }
    get unorderedAllRooms() {
        return Array.from(this.roomMap.values());
    }
    findRange(index) {
        return [
            Math.floor(index / WINDOW_SIZE) * WINDOW_SIZE,
            Math.floor((index + WINDOW_SIZE) / WINDOW_SIZE) * WINDOW_SIZE,
        ];
    }
    findIndexInRange(sourceIndex, range) {
        return sourceIndex - range[0];
    }
    getDefinitionFor(currentUserId) {
        const ranges = [];
        for (let i = 0; i <= Math.ceil(this.totalCount / WINDOW_SIZE); i++) {
            ranges.push([i * WINDOW_SIZE, (i + 1) * WINDOW_SIZE]);
        }
        return {
            filters: {
                is_dm: this.behaviour === ListBehaviour.DMsOnly,
                is_invite: this.behaviour === ListBehaviour.InvitedOnly,
            },
            sort: this.sort,
            required_state: V3Room_1.V3Room.REQUIRED_STATE.map(t => [t[0], t[1] === exports.CURRENT_USER_STATE_KEY ? currentUserId : t[1]]),
            timeline_limit: TIMELINE_LIMIT,
            rooms: ranges,
        };
    }
    getOrCreateRoom(client, room) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.roomMap.get(room.room_id)) {
                this.roomMap.set(room.room_id, new V3Room_1.V3Room(room.room_id));
            }
            const mapped = this.roomMap.get(room.room_id);
            if (room.required_state)
                yield mapped.updateState(client, room.required_state);
            if (room.timeline)
                yield mapped.updateTimeline(client, room.timeline);
            return mapped;
        });
    }
    getOrCreateRangeFromIndex(index) {
        const range = this.findRange(index);
        const inRangeIdx = this.findIndexInRange(index, range);
        let rooms = this.roomsByRange.get(JSON.stringify(range));
        if (!rooms) {
            rooms = new Array(range[1] - range[0]).fill(null);
            this.roomsByRange.set(JSON.stringify(range), rooms);
        }
        return { range, inRangeIdx, rooms };
    }
    processOperations(client, totalCount, ops) {
        return __awaiter(this, void 0, void 0, function* () {
            if (totalCount)
                this.totalCount = totalCount;
            for (const op of ops) {
                switch (op.op) {
                    case operations_1.Operation.Delete: {
                        const range = this.findRange(op.index);
                        const inRangeIdx = this.findIndexInRange(op.index, range);
                        const rooms = this.roomsByRange.get(JSON.stringify(range));
                        if (rooms) {
                            const [deletedRoom] = rooms.splice(inRangeIdx, 1);
                            this.roomMap.delete(deletedRoom.roomId);
                        }
                        break;
                    }
                    case operations_1.Operation.Insert: {
                        const info = this.getOrCreateRangeFromIndex(op.index);
                        info.rooms[info.inRangeIdx] = yield this.getOrCreateRoom(client, op.room);
                        break;
                    }
                    case operations_1.Operation.Update: {
                        const info = this.getOrCreateRangeFromIndex(op.index);
                        const room = info.rooms[info.inRangeIdx];
                        if (!room)
                            throw new Error("Failed to handle update operation: unknown room at index. Op: " + JSON.stringify(op));
                        if (op.room.required_state)
                            yield room.updateState(client, op.room.required_state);
                        if (op.room.timeline)
                            yield room.updateTimeline(client, op.room.timeline);
                        break;
                    }
                    case operations_1.Operation.Sync: {
                        const rooms = new Array(op.range[1] - op.range[0]).fill(null);
                        for (let i = 0; i < op.rooms.length; i++) {
                            if (!op.rooms[i])
                                continue;
                            rooms[i] = yield this.getOrCreateRoom(client, op.rooms[i]);
                        }
                        this.roomsByRange.set(JSON.stringify(op.range), rooms);
                        break;
                    }
                    case operations_1.Operation.Invalidate: {
                        this.roomsByRange.delete(JSON.stringify(op.range));
                        break;
                    }
                }
            }
        });
    }
}
exports.V3List = V3List;
